import { LOGBOOK_API } from "../actions/actionType";
import Super from "./super";
import moment from "moment";
const defaultState = {
  list: [],
  data: {},
  procedures: [],
  logbook_procedures: [],
  logbook_forms: [],
  logbook_form: null,
  logbook_manage: {},
  comments: [],
  logbook_diseases: [],
  diseases: [],
  institute: []
};
export default (state = defaultState, action) => {
  let reducer = new Super(state, action);
  switch (action.type) {
    case LOGBOOK_API.SELF.CLEAR.SUCCESS:
      return {
        ...state,
        data: {},
        logbook_procedures: [],
        logbook_forms: [],
        list: [],
        institute: [],
        procedures: [],
        diseases: []
      };
    case LOGBOOK_API.SELF.ALL.SUCCESS:
      return reducer.list();
    case LOGBOOK_API.INSTITUTE.ALL.SUCCESS:
      return reducer.list("institute");
    case LOGBOOK_API.SELF.DEL.SUCCESS:
      return reducer.delete();
    case LOGBOOK_API.PROCEDURE.ALL.SUCCESS:
      return reducer.list("procedures");
    case LOGBOOK_API.SELF.GET.SUCCESS:
      var newState = reducer.get();
      return {
        ...newState,
        files: (newState.data.files || []).map(v => ({
          uid: v.file.id,
          name: v.file.name,
          status: "done",
          response: { id: v.file.id },
          url: "/api/file/files/download/" + v.file.id
        })),
        logbook_procedures: newState.data.logbookProcedures,
        logbook_diseases: newState.data.logbookDiseases,
        logbook_forms: newState.data.logbookFormResponses
          ? newState.data.logbookFormResponses
          : [],
        comments: newState.data.logbookComments
      };
    case LOGBOOK_API.FILE.DEL.SUCCESS:
      return {
        ...state,
        files: state.files.filter(v => v.uid !== action.data)
      };
    case LOGBOOK_API.FILE.POST.SUCCESS:
      return {
        ...state,
        files: [
          ...state.files,
          {
            uid: action.data.id,
            name: action.data.name,
            status: "done",
            response: { id: action.data.id },
            url: "/api/file/files/download/" + action.data.id
          }
        ]
      };
    case LOGBOOK_API.SELF.POST.SUCCESS:
      return reducer.post();
    case LOGBOOK_API.SELF.PUT.SUCCESS:
      var newState = reducer.get();
      return {
        ...newState,
        files: (newState.data.files || []).map(v => ({
          uid: v.file.id,
          name: v.file.name,
          status: "done",
          response: { id: v.file.id },
          url: "/api/file/files/download/" + v.file.id
        })),
        logbook_procedures: newState.data.logbookProcedures,
        logbook_diseases: newState.data.logbookDiseases,
        logbook_forms: newState.data.logbookFormResponses
          ? newState.data.logbookFormResponses
          : [],
        comments: newState.data.logbookComments
      };
    case LOGBOOK_API.PROCEDURE.POST.SUCCESS:
      return reducer.insertOrUpdate("logbook_procedures");
    case LOGBOOK_API.LOGBOOK_FORM.GET.SUCCESS:
      return reducer.get("logbook_form");
    case LOGBOOK_API.LOGBOOK_FORM.POST.SUCCESS:
      return reducer.post("logbook_forms");
    case LOGBOOK_API.LOGBOOK_FORM.DEL.SUCCESS:
      return {
        ...state,
        logbook_forms: state.logbook_forms.filter(
          v => v.id.formResponseId !== action.data.formResponseId
        )
      };
    case LOGBOOK_API.LOGBOOK_MANAGE.ALL.SUCCESS:
      return {
        ...state,
        logbook_manage: action.data
          ? action.data.length > 0
            ? action.data[0]
            : {}
          : {}
      };
    case LOGBOOK_API.VALIDATE.PUT.SUCCESS:
      return reducer.put();
    case LOGBOOK_API.COMMENT.POST.SUCCESS:
      return {
        ...state,
        comments: [...state.comments, action.data]
      };
    case LOGBOOK_API.COMMENT.DEL.SUCCESS:
      return reducer.delete("comments");
    case LOGBOOK_API.DISEASE.ALL.SUCCESS:
      return reducer.list("diseases");
    default:
      return state;
  }
};
