import { MEDIC_API } from "../actions/actionType";
import Super from "./super";
import moment from "moment";
const defaultState = {
  list: [],
  data: {},
  patients: [],
  patient: {},
  followUps: [],
  followUp: {},
  stat: {},
  compare: {},
  headers: []
};

export default (state = defaultState, action) => {
  let reducer = new Super(state, action);
  switch (action.type) {
    case MEDIC_API.SELF.CLEAR.SUCCESS:
      return { ...state, followUp: {}, followUps: [], patients: [] };
    case MEDIC_API.FOLLOWUP.CLEAR.SUCCESS:
      return { ...state, followUp: {} };
    case MEDIC_API.PATIENT.ALL.SUCCESS:
      return reducer.list("patients");
    case MEDIC_API.PATIENT.CLEAR.SUCCESS:
      return { ...state, patients: [] };
    case MEDIC_API.PATIENT.GET.SUCCESS:
      return reducer.get("patient");

    case MEDIC_API.PATIENT.POST.SUCCESS:
      return reducer.post("patients");
    case MEDIC_API.PATIENT.PUT.SUCCESS:
      return reducer.put("patient", "patients");
    case MEDIC_API.PATIENT.DEL.SUCCESS:
      return reducer.put("patients");
    case MEDIC_API.FOLLOWUP.ALL.SUCCESS:
      return reducer.list("followUps");
    case MEDIC_API.FOLLOWUP.GET.SUCCESS:
      return reducer.get("followUp");
    case MEDIC_API.FOLLOWUP.DEL.SUCCESS:
      return reducer.delete("followUps");
    case MEDIC_API.FOLLOWUP.POST.SUCCESS:
      return { ...reducer.post("followUps"), followUp: action.data };
    case MEDIC_API.FOLLOWUP.PUT.SUCCESS:
      return reducer.put("followUp");
    case MEDIC_API.FILE.POST.SUCCESS:
      return {
        ...state,
        followUp: {
          ...state.followUp,
          files: [
            {
              fileId: action.data.file.id,
              filename: action.data.file.name,
              mime: action.data.file.mime
            },
            ...state.followUp.files
          ]
        }
      };
    case MEDIC_API.FILE.DEL.SUCCESS:
      return {
        ...state,
        followUp: {
          ...state.followUp,
          files: state.followUp.files.filter(v => v.fileId !== action.data)
        }
      };
    case MEDIC_API.FORM.DEL.SUCCESS:
      return {
        ...state,
        followUp: {
          ...state.followUp,
          formAnswers: state.followUp.formAnswers.filter(
            v => v.response.id !== action.data
          )
        }
      };
    case MEDIC_API.STAT_GROUP.GET.SUCCESS:
      return reducer.get("stat");
    case MEDIC_API.STAT.GET.SUCCESS:
      return action.compare
        ? reducer.get("compare")
        : { ...reducer.get("stat"), compare: {} };
    case MEDIC_API.HEADER.ALL.SUCCESS:
      return { ...state, headers: action.data };
    default:
      return state;
  }
};
