import { WFME_API } from "../actions/actionType";
import Super from "./super";
import moment from "moment";
const defaultState = {
  list: [],
  data: {},
  kpis: [],
  kpi: {},
  course: {},
  courses: [],
  offers: [],
  offer: {},
  answers: [],
  answer: {},
  outcomes: [],
  departments: [],
  files: [],
  curriculums: [],
  fileNames: [],
  responses: [],
  acadYears: [],
  outcome_forms: [],
  cucas: [],
  cucas_user: []
};

export default (state = defaultState, action) => {
  let reducer = new Super(state, action);
  switch (action.type) {
    case WFME_API.SELF.CLEAR.SUCCESS:
      return { ...state, offers: [], curriculums: [] };
    case WFME_API.FILE_NAME.ALL.SUCCESS:
      return reducer.list("fileNames");
    case WFME_API.CURRICULUM.ALL.SUCCESS:
      return reducer.list("curriculums");
    case WFME_API.KPI.ALL.SUCCESS:
      return reducer.list("kpis");
    case WFME_API.KPI.GET.SUCCESS:
      return reducer.get("kpi");
    case WFME_API.KPI.POST.SUCCESS:
      return reducer.post("kpis");
    case WFME_API.KPI.PUT.SUCCESS:
      return reducer.put("kpi", "kpis");
    case WFME_API.KPI.DEL.SUCCESS:
      return reducer.delete("kpis");
    case WFME_API.COURSE.ALL.SUCCESS:
      return reducer.list("courses");
    case WFME_API.COURSE.GET.SUCCESS:
      return reducer.get("course");
    case WFME_API.COURSE.POST.SUCCESS:
      return reducer.post("courses");
    case WFME_API.COURSE.PUT.SUCCESS:
      return reducer.put("course", "courses");
    case WFME_API.COURSE.DEL.SUCCESS:
      return reducer.delete("courses");
    case WFME_API.COURSE_OFFER.ALL.SUCCESS:
      return reducer.list("offers");
    case WFME_API.COURSE_OFFER.GET.SUCCESS:
      return {
        ...state,
        offer: action.data,
        responses: action.data.kpiAnswers
      };
    case WFME_API.COURSE_OFFER.POST.SUCCESS:
      return reducer.post("offers");
    case WFME_API.COURSE_OFFER.PUT.SUCCESS:
      return reducer.put("offer", "offers");
    case WFME_API.COURSE_OFFER.DEL.SUCCESS:
      return reducer.delete("offers");
    case WFME_API.ANSWER.ALL.SUCCESS:
      return reducer.list("answers");
    case WFME_API.ANSWER.GET.SUCCESS:
      return {
        ...reducer.get("answer"),
        answer: {
          ...action.data,
          evidences: (action.data.evidences ? action.data.evidences : []).map(
            v => ({
              uid: v.fileId,
              name: v.filename,
              status: "done",
              response: { id: v.fileId },
              url: "/api/file/files/download/" + v.fileId
            })
          )
        }
      };
    case WFME_API.ANSWER.POST.SUCCESS:
      return reducer.post("answers");
    case WFME_API.ANSWER.PUT.SUCCESS:
      return reducer.put("answer", "answers");
    case WFME_API.ANSWER.DEL.SUCCESS:
      return reducer.delete("answers");
    case WFME_API.OUTCOME.ALL.SUCCESS:
      return reducer.list("outcomes");
    case WFME_API.OUTCOME.PUT.SUCCESS:
      return {
        ...state,
        offer: {
          ...state.offer,
          outcomeAnswers:
            state.offer.outcomeAnswers.filter(
              v => v.outcome.id === action.data.outcome.id
            ).length > 0
              ? state.offer.outcomeAnswers.map(v =>
                  v.outcome.id === action.data.outcome.id
                    ? { ...action.data }
                    : { ...v }
                )
              : [...state.offer.outcomeAnswers, action.data]
        }
      };
    case WFME_API.DEPARTMENT.ALL.SUCCESS:
      return reducer.list("departments");
    case WFME_API.FILE.ALL.SUCCESS:
      return {
        ...state,
        files: action.data.map(v => ({
          uid: v.id,
          name: v.originalFilename,
          status: "done",
          response: { id: v.id },
          url: "/api/file/files/download/" + v.id
        }))
      };
    case WFME_API.FILE.POST.SUCCESS:
      return {
        ...state,
        files: [
          ...state.files,
          {
            uid: action.data.id,
            name: action.data.originalFilename,
            status: "done",
            response: { id: action.data.id },
            url: "/api/file/files/download/" + action.data.id
          }
        ]
      };
    case WFME_API.FILE.DEL.SUCCESS:
      return {
        ...state,
        files: state.files.filter(v => v.uid !== action.data)
      };
    case WFME_API.RESPONSE.POST.SUCCESS:
      return reducer.post("responses");
    case WFME_API.ACADYEAR.ALL.SUCCESS:
      return { ...state, acadYears: action.data };
    case WFME_API.OUTCOME_FORM.PUT.SUCCESS:
      return {
        ...state,
        outcome_forms: state.outcome_forms.map(v =>
          v
            ? v.id.outcomeId === action.data.id.outcomeId
              ? { ...action.data }
              : { ...v }
            : null
        )
      };
    case WFME_API.OUTCOME_FORM.ALL.SUCCESS:
      return { ...state, outcome_forms: action.data };
    case WFME_API.CUCAS.ALL.SUCCESS:
      return reducer.list("cucas", "courseid");
    case WFME_API.CUCAS_USER.ALL.SUCCESS:
      return { ...state, cucas_user: action.data };
    default:
      return state;
  }
};
