import { call, put, takeEvery } from "redux-saga/effects";
import {
  COMPONENT,
  LOGIN,
  LOGOUT,
  REDIRECT_TO
} from "../actions/actionType.js";
import auth from "../services/auth";
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: ""
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loginFlow(actions) {
  try {
    yield put({ type: LOGIN.SENDING, sending: true });
    const { username, password } = actions.payload;
    try {
      const response = yield call(auth.login, username, password);
      yield put({ type: LOGIN.SUCCESS });
      yield put({ type: REDIRECT_TO, redirectTo: "/", query: {} });
    } catch (error) {
      console.log(error);
      yield put({
        type: LOGIN.FAILURE,
        error: error.response ? error.response.data.message : ""
      });
    } finally {
      yield put({ type: LOGIN.SENDING, sending: false });
    }
  } catch (e) {
    yield call(callback, false, e);
  }
}
function* logoutFlow(actions) {
  yield put({ type: LOGOUT.SENDING, sending: true });
  try {
    const response = yield call(auth.logout);
    yield put({ type: LOGOUT.SUCCESS });
    yield put({ type: REDIRECT_TO, redirectTo: "/login", query: {} });
  } catch (error) {
    yield put({ type: LOGOUT.FAILURE, error: error.response.data.message });
  } finally {
    yield put({ type: LOGOUT.SENDING, sending: false });
  }
}

export function* watchAuth() {
  yield takeEvery(LOGIN.REQUEST, loginFlow);
  yield takeEvery(LOGOUT.REQUEST, logoutFlow);
}
