import {
  COMPONENT,
  GET,
  POST,
  PUT,
  DEL,
  ALL,
  CLEAR,
  SEARCH,
  CONFERENCE_REQUEST,
  CONFERENCE_API,
  SELF,
  DETAIL,
  ATTENDANCE,
  ME,
  ATTENDED,
  SESSION,
  USER_API,
  CONFERENCE,
  TYPE,
  UPCOMING
} from "../actions/actionType.js";
import { put, fork, call, take, takeEvery, throttle } from "redux-saga/effects";
import graphql from "../services/graphql";
import auth from "../services/auth";
import roles from "../roles";
import query from "query";
const url = "conference";
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error.response.data.message
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* priority() {
  yield put({
    type: COMPONENT.CALLBACK_PRIORITY
  });
}
function* clearPriority() {
  yield put({
    type: COMPONENT.CALLBACK_CLEAR
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield fork(clear, actions);
    default:
      null;
  }
}
function* clear(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        return yield put({
          type: CONFERENCE_API.SELF.CLEAR.SUCCESS
        });
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        var services = roles.permission(me.data.data.me.roles, [
          "ADMIN",
          "INSTITUTE"
        ])
          ? "allConferences"
          : "involveConferences";
        console.log(services);
        console.log(actions.obj.isMe);
        if (actions.obj) {
          if (actions.obj.isMe) {
            services = "involveConferences";
          }
        }
        var response = yield call(
          graphql.search,
          services,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          roles.permission(me.data.data.me.roles, ["INSTITUTE"])
            ? {
                ...actions,
                item: {
                  ...actions.item,
                  instituteId: { $oid: me.data.data.me.institute.id }
                }
              }
            : actions
        );
        yield put({
          type: CONFERENCE_API.SELF.ALL.SUCCESS,
          data: response.data.data[services]
        });
        if (response.data.data[services].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case UPCOMING:
        yield call(loading);
        var services = "meConferencesBefore";
        var response = yield call(
          graphql.search,
          services,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          actions
        );
        yield put({
          type: CONFERENCE_API.UPCOMING.ALL.SUCCESS,
          data: response.data.data[services]
        });
        if (response.data.data[services].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case ATTENDED:
        yield call(loading);
        var services = "meAttendedConferences";
        var response = yield call(
          graphql.search,
          services,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          actions
        );
        yield put({
          type: CONFERENCE_API.ATTENDED.ALL.SUCCESS,
          data: response.data.data[services]
        });
        if (response.data.data[services].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case CONFERENCE:
        yield call(loading);
        var services = "meConferences";
        var response = yield call(
          graphql.search,
          services,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          actions
        );
        yield put({
          type: CONFERENCE_API.CONFERENCE.ALL.SUCCESS,
          data: response.data.data[services]
        });
        let responses = response.data.data[services];
        if (responses && responses.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case TYPE:
        yield call(loading);
        var me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        var services = "conferenceTypes";
        var response = yield call(
          graphql.search,
          services,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          {
            ...actions,
            item: { specialtyId: { $oid: me.data.data.me.specialty.id } }
          }
        );
        yield put({
          type: CONFERENCE_API.TYPE.ALL.SUCCESS,
          data: response.data.data[services]
        });
        if (response.data.data[services].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case DETAIL:
        var response = yield call(
          graphql.get,
          `conference(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: CONFERENCE_API.DETAIL.GET.SUCCESS,
          data: response.data.data.conference
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* post(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(graphql.post, "Conference", url, actions);
        yield put({
          type: CONFERENCE_API.SELF.POST.SUCCESS,
          data: response.data.data.insertConference
        });
        return history.back();
      case TYPE:
        yield call(loading);
        var response = yield call(graphql.post, "ConferenceType", url, actions);
        yield put({
          type: CONFERENCE_API.TYPE.POST.SUCCESS,
          data: response.data.data.insertConferenceType
        });
        yield call(complete);
        return history.back();
      case ATTENDANCE:
        try {
          yield call(loading);
          if (actions.id) {
            try {
              const login = yield call(
                auth.login,
                actions.id.username,
                actions.id.password
              );
              var me = yield call(graphql.get, "me", "account", {
                ...actions,
                query: query.auth.me
              });
              yield put({
                type: USER_API.ME.GET.SUCCESS,
                data: me.data.data.me
              });
              actions.item.id["userId"] = me.data.data.me.username;
            } catch (e) {
              console.log(e);
            }
          }
          var response = yield call(graphql.post, "Attendance", url, actions);
          if (response.data.data.insertAttendance) {
            yield put({
              type: CONFERENCE_API.ATTENDANCE.POST.SUCCESS,
              data: response.data.data.insertAttendance
            });
          } else {
            yield call(callback, false, {
              response: {
                data: {
                  message:
                    response.data.errors.length > 0
                      ? response.data.errors[0].message
                      : ""
                }
              }
            });
          }

          yield put({
            type: CONFERENCE_REQUEST,
            doc: DETAIL,
            api: GET,
            item: {},
            id: actions.item.id.conferenceId,
            query: query.conference.data
          });
          return yield call(callback, true);
        } catch (e) {
          console.log(e);
        }
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(graphql.put, "Conference", url, actions);
        yield put({
          type: CONFERENCE_API.SELF.PUT.SUCCESS,
          data: response.data.data.updateConference
        });
        yield call(callback, true);
        return history.back();
      case TYPE:
        var response = yield call(graphql.put, "ConferenceType", url, actions);
        yield put({
          type: CONFERENCE_API.TYPE.PUT.SUCCESS,
          data: response.data.data.updateConferenceType
        });
        yield call(callback, true);
        return history.back();
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}

function* del(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(graphql.delete, "Conference", url, actions);
        yield put({
          type: CONFERENCE_API.SELF.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return history.back();
      case TYPE:
        yield call(loading);
        var response = yield call(
          graphql.delete,
          "ConferenceType",
          url,
          actions
        );
        yield put({
          type: CONFERENCE_API.TYPE.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return history.back();
      case ATTENDANCE:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation{
  deleteAttendance(id:{conferenceId:"${actions.id.conferenceId}",userId:"${
            actions.id.userId
          }"})
}`,
          {},
          url,
          actions
        );
        yield put({
          type: CONFERENCE_API.ATTENDANCE.DEL.SUCCESS,
          data: actions.id
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
export function* watchConference() {
  yield takeEvery(CONFERENCE_REQUEST, request);
}
