import { call, fork, put, takeEvery } from "redux-saga/effects";
import {
  ALL,
  CLEAR,
  COMPONENT,
  DEL,
  EXAM_API,
  EXAM_REQUEST,
  FORM_OPNOTE,
  FORM_QUESTION,
  GET,
  MCQ,
  MEQ,
  OPNOTE,
  ORAL,
  POST,
  PROBLEM,
  PUT,
  QUESTION,
  SEARCH,
  SELF,
  SEQ,
  STUDENT,
  USER,
  VALIDATE
} from "../actions/actionType.js";
import graphql from "../services/graphql";
const url = "exam";
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error.response.data.message
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* priority() {
  yield put({
    type: COMPONENT.CALLBACK_PRIORITY
  });
}
function* clearPriority() {
  yield put({
    type: COMPONENT.CALLBACK_CLEAR
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield fork(clear, actions);
    default:
      null;
  }
}
function* clear(actions) {
  try {
    switch (actions.doc) {
      case USER:
        return yield put({
          type: EXAM_API.USER.CLEAR.SUCCESS
        });
      case SELF:
        return yield put({
          type: EXAM_API.SELF.CLEAR.SUCCESS
        });
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var service = "examinations";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          actions
        );
        yield put({
          type: EXAM_API.SELF.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case USER:
        yield call(loading);
        var service = "examinationStudents";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 20) : 20,
          url,
          actions
        );
        if (actions.obj.skip === 0) {
          yield put({
            type: EXAM_API.STUDENT.CLEAR.SUCCESS
          });
        }
        yield put({
          type: EXAM_API.USER.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case PROBLEM:
        yield call(loading);
        var service = "examinationStudents";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 20) : 20,
          url,
          actions
        );
        yield put({
          type: EXAM_API.USER.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case QUESTION:
        yield call(loading);
        var service = "questions";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          actions
        );
        yield put({
          type: EXAM_API.QUESTION.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }

      case STUDENT:
        yield call(loading);
        var service = "committeeExaminationStudents";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 20) : 20,
          url,
          actions
        );
        yield put({
          type: EXAM_API.STUDENT.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(
          graphql.get,
          `examination(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: EXAM_API.SELF.GET.SUCCESS,
          data: response.data.data.examination
        });
        return yield call(complete);
      case USER:
        try {
          var response = yield call(
            graphql.get,
            `examinationStudent(id:{studentId:"${
              actions.id.studentId
            }",examinationId:"${actions.id.examId}"})`,
            url,
            actions
          );
          yield put({
            type: EXAM_API.USER.GET.SUCCESS,
            data: response.data.data.examinationStudent
          });
          return yield call(complete);
        } catch (e) {
          console.log(e);
        }
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* post(actions) {
  try {
    switch (actions.doc) {
      case ORAL:
        yield call(loading);
        var response = yield call(
          graphql.post,
          "OralExamination",
          url,
          actions
        );
        yield put({
          type: EXAM_API.ORAL.POST.SUCCESS,
          data: response.data.data.insertOralExamination
        });
        return history.back();
      case SEQ:
        yield call(loading);
        var response = yield call(
          graphql.post,
          "SectionHeaderItem",
          url,
          actions
        );
        yield put({
          type: EXAM_API.SEQ.POST.SUCCESS,
          data: response.data.data.insertSectionHeaderItem
        });
        return history.back();
      case MEQ:
        yield call(loading);
        var response = yield call(
          graphql.post,
          "MultipleChoiceItem",
          url,
          actions
        );
        yield put({
          type: EXAM_API.MEQ.POST.SUCCESS,
          data: response.data.data.insertMultipleChoiceItem
        });
        return history.back();
      case MCQ:
        yield call(loading);
        var response = yield call(graphql.post, "McqExamination", url, actions);
        yield put({
          type: EXAM_API.MCQ.POST.SUCCESS,
          data: response.data.data.insertMcqExamination
        });
        return history.back();
      case QUESTION:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation{
        insertOralExaminationQuestions(examinationId:"${
          actions.id
        }",questionIds:${actions.item}){
          exception
        }
      }`,
          {},
          url,
          actions
        );
        return history.back();
      case USER:
        var response = yield call(
          graphql.graph,
          `mutation {
            insertOralExaminationStudents(examinationId:"${
              actions.item.examId
            }",studentIds:${JSON.stringify(actions.item.students)}){
              document {
                student {
                  username
                  titleTh
                  titleEn
                  firstNameTh
                  lastNameTh
                  firstNameEn
                  lastNameEn
                  gender
                  yearEnrolled
                }
              }
              exception
            }
          }`,
          {},
          url,
          actions
        );
        yield put({
          type: EXAM_API.USER.POST.SUCCESS,
          data: response.data.data.insertOralExaminationStudents
        });
        if (actions.obj) {
          return history.back();
        } else {
          return yield call(callback, true);
        }

      case OPNOTE:
        var response = yield call(
          graphql.post,
          `OralExaminationOpnote`,
          url,
          actions
        );
        yield put({
          type: EXAM_API.OPNOTE.POST.SUCCESS,
          data: response.data.data.insertOralExaminationOpnote
        });
        yield call(callback, true);
        return history.back();
      case FORM_OPNOTE:
        var response = yield call(
          graphql.graph,
          `mutation{
            insertOralOpnoteResponse(data:{examinationId:"${
              actions.id.examinationId
            }",formResponseId:"${actions.id.formResponseId}",studentId:"${
            actions.id.studentId
          }",oralExaminationOpnoteId:"${
            actions.id.oralExaminationOpnoteId
          }",committeeId:"${actions.id.committeeId}"}){
              ${actions.query.map(v => v + " ")}
            }
          }`,
          {},
          url,
          actions
        );
        yield put({
          type: EXAM_API.FORM_OPNOTE.POST.SUCCESS,
          data: response.data.data.insertOralOpnoteResponse
        });
        return yield call(callback, true);
      case FORM_QUESTION:
        var response = yield call(
          graphql.graph,
          `mutation{
            insertOralQuestionResponse(data:{examinationId:"${
              actions.id.examinationId
            }",formResponseId:"${actions.id.formResponseId}",studentId:"${
            actions.id.studentId
          }",committeeId:"${actions.id.committeeId}",questionId:"${
            actions.id.questionId
          }"}){
                    ${actions.query.map(v => v + " ")}
          }
        }`,
          {},
          url,
          actions
        );
        yield put({
          type: EXAM_API.FORM_QUESTION.POST.SUCCESS,
          data: response.data.data.insertOralQuestionResponse
        });
        yield call(callback, true);

      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  try {
    switch (actions.doc) {
      case VALIDATE:
        var response = yield call(
          graphql.graph,
          `mutation{
        examinationResponseValidate(id:"${actions.id}",status:${
            actions.item.status
          }){
          ${actions.query.map(v => v + " ")}
        }
      }`,
          {},
          url,
          actions
        );
        yield put({
          type: EXAM_API.VALIDATE.PUT.SUCCESS,
          data: response.data.data.examinationResponseValidate
        });
        return yield call(callback, true);
      case SELF:
        var response = yield call(graphql.put, "OralExamination", url, actions);
        yield put({
          type: EXAM_API.SELF.PUT.SUCCESS,
          data: response.data.data.updateOralExamination
        });
        yield call(callback, true);
        return history.back();
      case USER:
        var response = yield call(
          graphql.graph,
          `mutation updateOralExaminationStudent($data: OralExaminationStudentInput) {
            updateOralExaminationStudent(data: $data, id: {studentId:"${
              actions.id.studentId
            }",examinationId:"${actions.id.examinationId}"}) {
              ${actions.query.map(v => v + " ")}
            }
          }
          `,
          { data: actions.item },
          url,
          actions
        );
        yield put({
          type: EXAM_API.USER.PUT.SUCCESS,
          data: response.data.data.updateOralExaminationStudent
        });
        return yield call(callback, true);

      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}

function* del(actions) {
  try {
    switch (actions.doc) {
      case FORM_QUESTION:
        var response = yield call(
          graphql.delete,
          "OralQuestionResponse",
          url,
          actions
        );
        yield put({
          type: EXAM_API.FORM_QUESTION.DEL.SUCCESS,
          data: actions.id,
          questionId: actions.obj.questionId
        });
        return yield call(callback, true);
      case FORM_OPNOTE:
        var response = yield call(
          graphql.delete,
          "OralOpnoteResponse",
          url,
          actions
        );
        yield put({
          type: EXAM_API.FORM_OPNOTE.DEL.SUCCESS,
          data: actions.id,
          opnoteId: actions.obj.opnoteId
        });
        return yield call(callback, true);
      case OPNOTE:
        var response = yield call(
          graphql.delete,
          "OralExaminationOpnote",
          url,
          actions
        );
        yield put({
          type: EXAM_API.OPNOTE.DEL.SUCCESS,
          data: actions.id
        });
        return yield call(callback, true);
      case QUESTION:
        var response = yield call(
          graphql.graph,
          `mutation{
            deleteOralExaminationQuestion(id:{examinationId:"${
              actions.obj.examinationId
            }",questionId:"${actions.obj.questionId}"})}`,
          {},
          url,
          actions
        );
        yield put({
          type: EXAM_API.QUESTION.DEL.SUCCESS,
          data: actions.obj.questionId
        });
        return yield call(callback, true);
      case USER:
        var response = yield call(
          graphql.graph,
          `mutation{
            deleteOralExaminationStudent(id:{studentId:"${
              actions.obj.studentId
            }",examinationId:"${actions.obj.examinationId}"})}`,
          {},
          url,
          actions
        );
        yield put({
          type: EXAM_API.USER.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return history.back();
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
export function* watchExam() {
  yield takeEvery(EXAM_REQUEST, request);
}
