import query from "query";
import { call, fork, put, takeEvery } from "redux-saga/effects";
import {
  ALL,
  ANSWER,
  CHECKBOX,
  CHECKBOX_GRID,
  CLEAR,
  COMPONENT,
  CONDITION,
  DATE,
  DATETIME,
  DEL,
  DROPDOWN,
  EQUATION,
  EXAM_REQUEST,
  EXPORT_CSV,
  FORM,
  FORM_API,
  FORM_OPNOTE,
  FORM_QUESTION,
  FORM_REQUEST,
  GET,
  GRID,
  IMAGE,
  ITEM,
  LIST,
  MEDIC_REQUEST,
  MOVE,
  MULTIPLE_CHOICE,
  OWN,
  POST,
  PUT,
  REDIRECT_TO,
  RESPONSE,
  SCALE,
  SEARCH,
  SECTION_HEADER,
  SELF,
  STAT,
  STAT_GROUP,
  TAG,
  TEXT,
  TIME,
  TYPE,
  VIDEO,
  WFME_REQUEST,
  STUDENT,
  RESPONSED_STUDENT,
  SPECIALTY_FORM,
  INSTITUTE,
  EMAIL
} from "../actions/actionType.js";
import roles from "../roles";
import auth from "../services/auth";
import graphql from "../services/graphql";
const url = "form";
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error.response.data.message
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* priority() {
  yield put({
    type: COMPONENT.CALLBACK_PRIORITY
  });
}
function* clearPriority() {
  yield put({
    type: COMPONENT.CALLBACK_CLEAR
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield fork(clear, actions);
    default:
      null;
  }
}
function* clear(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        return yield put({
          type: FORM_API.SELF.CLEAR.SUCCESS,
          data: []
        });
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "forms",
          actions.obj ? actions.obj.skip : 0,
          20,
          url,
          actions
        );
        var data = response.data.data.forms;
        yield put({
          type: FORM_API.SELF.ALL.SUCCESS,
          data: data
        });
        if (data.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case OWN:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "myForms",
          actions.obj ? actions.obj.skip : 0,
          20,
          url,
          actions
        );
        var data = response.data.data.myForms;
        yield put({
          type: FORM_API.SELF.ALL.SUCCESS,
          data: data
        });
        if (data.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case RESPONSE:
        yield call(loading);
        var me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        if (roles.permission(me.data.data.me.roles, ["INSTITUTE"])) {
          var response = yield call(
            graphql.graph,
            `query formResponses($query:Object){formResponses(size:10,page:${
              actions.obj ? actions.obj.skip : 0
            },query:$query,instituteId:"${
              me.data.data.me.institute.id
            }"){${actions.query.map(v => v + " ")}}}`,
            { query: actions.item },
            url,
            actions
          );
          var data = response.data.data.formResponses;
          yield put({
            type: FORM_API.RESPONSE.ALL.SUCCESS,
            data: data
          });
          if (data.length == 0) {
            return yield call(outOfRange);
          } else {
            return yield call(complete);
          }
        }
        var services = roles.permission(me.data.data.me.roles, ["INSTITUTE"])
          ? "formResponses"
          : "meFormResponses";
        var response = yield call(
          graphql.search,
          services,
          actions.obj ? actions.obj.skip : 0,
          20,
          url,
          actions
        );
        var data = response.data.data[services];
        yield put({
          type: FORM_API.RESPONSE.ALL.SUCCESS,
          data: data
        });
        if (data.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case EQUATION:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "allEvaluations",
          actions.obj ? actions.obj.skip : 0,
          20,
          url,
          actions
        );
        var data = response.data.data.allEvaluations;
        yield put({
          type: FORM_API.EQUATION.ALL.SUCCESS,
          data: data
        });
        if (data.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case TAG:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `{itemTags(formId:"${actions.id}")}`,
          {},
          url,
          actions
        );
        yield put({
          type: FORM_API.TAG.ALL.SUCCESS,
          data: response.data.data.itemTags
        });
        return yield call(complete);

      case STAT_GROUP:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `{statFormResponseGroupByDatetime(formId:"${actions.id}",inputs:${
            actions.item
          }){${actions.query.map(v => v + " ")}}}`,
          {},
          url,
          actions
        );
        yield put({
          type: FORM_API.STAT_GROUP.ALL.SUCCESS,
          data: response.data.data.statFormResponseGroupByDatetime
        });
        return yield call(complete);
      case RESPONSED_STUDENT:
        yield call(loading);
        var me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        var response = yield call(
          graphql.graph,
          `{formResponses(sortBy:"requestUserId",query:{responseUserId:"${
            me.data.data.me.username
          }"}){
            form{
              title
            }
            responseUser{
              username
              firstNameTh
              lastNameTh
            }
            requestUser{
              username
              firstNameTh
              lastNameTh
            }
          }}`,
          {},
          url,
          actions
        );
        yield put({
          type: FORM_API.RESPONSED_STUDENT.ALL.SUCCESS,
          data: response.data.data.formResponses
        });
        return yield call(complete);
      case EXPORT_CSV:
        try {
          yield call(loading);
          var response = yield call(
            graphql.postRest,
            `/form/forms/${actions.id}/answer/csv`,
            actions.item
          );
          yield put({
            type: FORM_API.EXPORT_CSV.ALL.SUCCESS,
            data: response.data
          });
          return yield call(complete);
        } catch (e) {
          console.log(e);
        }
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(
          graphql.get,
          `form(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: FORM_API.SELF.GET.SUCCESS,
          data: response.data.data.form
        });
        return yield call(complete);
      case RESPONSE:
        yield call(loading);
        yield call(clearPriority);
        var response = yield call(
          graphql.get,
          `formResponse(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: FORM_API.RESPONSE.GET.SUCCESS,
          data: response.data.data.formResponse
        });
        //Form surverant
        if (
          response.data.data.formResponse.formId === "5c627cce9cc04500014026e8"
        ) {
          var medic = yield call(graphql.search, "followUps", 0, 100, "medic", {
            ...actions,
            query: [
              `formAnswers{
      response{
        id
        formId
      }
    }`
            ],
            item: { patientId: actions.obj.patientId }
          });
          var forms = medic.data.data.followUps;
          var arr = [];
          for (var i in forms) {
            for (var j in forms[i].formAnswers) {
              if (
                forms[i].formAnswers[j].response.formId ===
                "5c627cce9cc04500014026e8"
              ) {
                arr.push(forms[i].formAnswers[j]);
              }
            }
          }
          if (arr.length > 1) {
            var index = -1;
            for (var i = 0; i < arr.length; i++) {
              if (arr[i].response.id === actions.id) {
                index = i;
              }
            }
            if (index < arr.length) {
              response = yield call(
                graphql.get,
                `formResponse(id:"${arr[index + 1].response.id}")`,
                url,
                actions
              );
              yield put({
                type: FORM_API.RESPONSE_RECENT.GET.SUCCESS,
                data: response.data.data.formResponse
              });
            }
          }
        }
        yield call(complete);
        return yield call(priority);

      case EQUATION:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `evaluation(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: FORM_API.EQUATION.GET.SUCCESS,
          data: response.data.data.evaluation
        });
        return yield call(complete);
      case STUDENT:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `allStatFormResponseGroupByStatus(userId:"${actions.obj.username}")`,
          url,
          actions
        );
        yield put({
          type: FORM_API.STUDENT.GET.SUCCESS,
          data: response.data.data.allStatFormResponseGroupByStatus
        });
        return yield call(complete);
      case STAT:
        yield call(loading);
        const me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        var services = roles.permission(me.data.data.me.roles, ["ADMIN"])
          ? "allStatFormResponseGroupByStatus"
          : roles.permission(me.data.data.me.roles, ["TEACHER"])
          ? "statFormResponseGroupByStatus"
          : "meStatFormResponseGroupByStatus";
        if (services == `meStatFormResponseGroupByStatus`) {
          var response = yield call(graphql.get, services, url, actions);
          yield put({
            type: FORM_API.STAT.GET.SUCCESS,
            data: response.data.data.meStatFormResponseGroupByStatus
          });
          return yield call(complete);
        } else if (services == `statFormResponseGroupByStatus`) {
          var response = yield call(
            graphql.get,
            actions.obj
              ? actions.obj.username
                ? `statFormResponseGroupByStatus(userId:"${
                    actions.obj.username
                  }")`
                : services
              : services,
            url,
            actions
          );
          yield put({
            type: FORM_API.STAT.GET.SUCCESS,
            data: response.data.data.statFormResponseGroupByStatus
          });
          return yield call(complete);
        } else if (services == "allStatFormResponseGroupByStatus") {
          var response = yield call(
            graphql.get,
            `allStatFormResponseGroupByStatus (specialtyId:"${
              me.data.data.me.specialty.id
            }"
             ${
               actions.obj
                 ? actions.obj.username
                   ? actions.obj.username != ""
                     ? `, userId:"${actions.obj.username}"`
                     : ""
                   : ""
                 : ""
             }
             ${
               actions.obj
                 ? actions.obj.instituteId != ""
                   ? `,instituteId:"${actions.obj.instituteId}"`
                   : ""
                 : ""
             })`,
            url,
            actions
          );
          yield put({
            type: FORM_API.STAT.GET.SUCCESS,
            data: response.data.data.allStatFormResponseGroupByStatus
          });
          return yield call(complete);
        }
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* post(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation initForm($data:FormInput){initForm(data:$data${
            actions.obj ? "," + actions.obj : ""
          }){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url
        );
        yield put({
          type: FORM_API.SELF.POST.SUCCESS,
          data: response.data.data.initForm
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: actions.id
            ? "/form/edit?id=" +
              response.data.data.initForm.id +
              "&kpiId=" +
              actions.id
            : "/form/edit?id=" + response.data.data.initForm.id
        });
      case TEXT:
        var response = yield call(
          graphql.graph,
          `mutation insertTextItemBefore($data:TextItemInput){insertTextItemBefore(data:$data,formId: "${
            actions.obj.formId
          }", beforeItemId: "${actions.obj.beforeItemId}"){${actions.query.map(
            v => v + " "
          )}}}`,
          { data: actions.item },
          url
        );
        yield put({
          type: FORM_API.TEXT.POST.SUCCESS,
          data: response.data.data.insertTextItemBefore,
          beforeItemId: actions.obj.beforeItemId
        });
        return yield call(complete, true);
      case RESPONSE:
        var response = yield call(
          graphql.graph,
          `mutation insertFormResponse($data:FormResponseInput){insertFormResponse(data:$data,formId:"${
            actions.obj.formId
          }"){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url
        );
        yield put({
          type: FORM_API.RESPONSE.POST.SUCCESS,
          data: response.data.data.insertFormResponse
        });
        if (actions.obj.followupId) {
          yield put({
            type: MEDIC_REQUEST,
            doc: FORM,
            api: POST,
            item: {
              id: actions.obj.followupId,
              responseId: response.data.data.insertFormResponse.id,
              requestUserId: actions.obj.patientId
            },
            id: null,
            query: query.medic.followUps
          });
        } else if (actions.obj.kpiAnswerId) {
          yield put({
            type: WFME_REQUEST,
            doc: ANSWER,
            api: PUT,
            item: { responseId: response.data.data.insertFormResponse.id },
            id: actions.obj.kpiAnswerId,
            query: query.wfme.answer,
            obj: { responseId: response.data.data.insertFormResponse.id }
          });
        } else if (actions.obj.questionId) {
          yield put({
            type: EXAM_REQUEST,
            doc: FORM_QUESTION,
            api: POST,
            item: {},
            id: {
              examinationId: actions.obj.examinationId,
              formResponseId: response.data.data.insertFormResponse.id,
              studentId: actions.obj.studentId,
              committeeId: actions.obj.committeeId,
              questionId: actions.obj.questionId
            },
            query: query.exam.formQuestion
          });
        } else if (actions.obj.oralExaminationOpnoteId) {
          //Exam
          yield put({
            type: EXAM_REQUEST,
            doc: FORM_OPNOTE,
            api: POST,
            item: {},
            id: {
              examinationId: actions.obj.examinationId,
              formResponseId: response.data.data.insertFormResponse.id,
              studentId: actions.obj.studentId,
              committeeId: actions.obj.committeeId,
              oralExaminationOpnoteId: actions.obj.oralExaminationOpnoteId
            },
            query: query.exam.formOpnote
          });
        }
        yield call(complete, true);
        if (actions.obj.committeeId) {
          return;
        }
        return yield put({
          type: REDIRECT_TO,
          redirectTo: actions.obj.redirectTo
            ? actions.obj.redirectTo +
              "?id=" +
              response.data.data.insertFormResponse.id +
              "&formId=" +
              response.data.data.insertFormResponse.form.id
            : actions.obj.public
            ? "/form/do/form?id=" +
              response.data.data.insertFormResponse.id +
              "&formId=" +
              response.data.data.insertFormResponse.form.id
            : actions.obj.kpiAnswerId
            ? "/form/do?id=" +
              response.data.data.insertFormResponse.id +
              "&formId=" +
              response.data.data.insertFormResponse.form.id +
              "&kpiAnswerId=" +
              actions.obj.kpiAnswerId
            : "/form/do?id=" +
              response.data.data.insertFormResponse.id +
              "&formId=" +
              response.data.data.insertFormResponse.form.id
        });

      case ITEM:
        var response = yield call(
          graphql.graph,
          `mutation insertOrUpdate${actions.obj.type}ItemResponse($data:${
            actions.obj.type
          }ItemResponseInput){insertOrUpdate${
            actions.obj.type
          }ItemResponse(data:$data,id: {formResponseId: "${
            actions.obj.formResponseId
          }", itemId: "${actions.obj.formId}"}){${actions.query.map(
            v => v + " "
          )}}}`,
          { data: actions.item },
          url,
          actions
        );
        yield put({
          type: FORM_API.ITEM.POST.SUCCESS,
          data:
            response.data.data[`insertOrUpdate${actions.obj.type}ItemResponse`]
        });
        return yield call(complete, true);
      case SECTION_HEADER:
        var response = actions.obj.pageBreakItemId
          ? yield call(
              graphql.graph,
              `mutation {insertSectionAfterPageBreak(formId: "${
                actions.obj.formId
              }", pageBreakItemId: "${
                actions.obj.pageBreakItemId
              }"){${actions.query.map(v => v + " ")}}}`,
              { data: actions.item },
              url
            )
          : yield call(
              graphql.graph,
              `mutation {insertSection(formId: "${
                actions.obj.formId
              }"){${actions.query.map(v => v + " ")}}}`,
              { data: actions.item },
              url
            );
        yield put({
          type: FORM_API.SELF.GET.SUCCESS,
          data: actions.obj.pageBreakItemId
            ? response.data.data.insertSectionAfterPageBreak
            : response.data.data.insertSection
        });
        return yield call(complete, true);
      case EQUATION:
        var response = yield call(
          graphql.graph,
          `mutation insertEvaluation($data:EvaluationInput){insertEvaluation(data:$data,formId:"${
            actions.obj.formId
          }"){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url,
          actions
        );
        yield put({
          type: FORM_API.EQUATION.POST.SUCCESS,
          data: response.data.data.insertEvaluation
        });
        yield call(complete, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo:
            "/form/edit?id=" + actions.obj.formId + "&menu=" + "equation"
        });
      case CONDITION:
        yield call(loading);
        var response = yield call(graphql.post, "Condition", url, actions);
        yield put({
          type: FORM_API.CONDITION.POST.SUCCESS,
          data: response.data.data.insertCondition
        });
        return yield call(callback, true);
      case EMAIL:
        var response = yield call(
          graphql.graph,
          `mutation{
  sendEmailFormResponse(id:"${actions.id}",email:"${actions.item.email}",url:"${
            actions.item.url
          }")
}`,
          { data: actions.item },
          url
        );
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(graphql.put, "Form", url, actions);
        yield put({
          type: FORM_API.SELF.PUT.SUCCESS,
          data: response.data.data.updateForm
        });
        return yield call(callback, true);
      case TEXT:
        var response = yield call(graphql.put, "TextItem", url, actions);
        yield put({
          type: FORM_API.TEXT.PUT.SUCCESS,
          data: response.data.data.updateTextItem
        });
        return yield call(complete, true);
      case LIST:
        var response = yield call(graphql.put, "ListItem", url, actions);
        yield put({
          type: FORM_API.LIST.PUT.SUCCESS,
          data: response.data.data.updateListItem
        });
        return yield call(complete, true);
      case SECTION_HEADER:
        var response = yield call(
          graphql.put,
          "SectionHeaderItem",
          url,
          actions
        );
        yield put({
          type: FORM_API.SECTION_HEADER.PUT.SUCCESS,
          data: response.data.data.updateSectionHeaderItem
        });
        return yield call(complete, true);
      case TYPE:
        var response = yield call(
          graphql.graph,
          `mutation{changeItemType(itemId:"${actions.item.itemId}",type:${
            actions.item.type
          }){
          ${[`id`, `title`, `type`].map(v => v + " ")}
        }}`,
          {},
          url
        );
        yield put({
          type: FORM_API.TYPE.PUT.SUCCESS,
          data: response.data.data.changeItemType
        });
        return yield call(complete, true);
      case CHECKBOX:
        var response = yield call(graphql.put, "CheckboxItem", url, actions);
        yield put({
          type: FORM_API.CHECKBOX.PUT.SUCCESS,
          data: response.data.data.updateCheckboxItem
        });
        return yield call(complete, true);
      case MULTIPLE_CHOICE:
        var response = yield call(
          graphql.put,
          "MultipleChoiceItem",
          url,
          actions
        );
        yield put({
          type: FORM_API.MULTIPLE_CHOICE.PUT.SUCCESS,
          data: response.data.data.updateMultipleChoiceItem
        });
        return yield call(complete, true);
      case DATE:
        var response = yield call(graphql.put, "DateItem", url, actions);
        yield put({
          type: FORM_API.DATE.PUT.SUCCESS,
          data: response.data.data.updateDateItem
        });
        return yield call(complete, true);
      case DATETIME:
        var response = yield call(graphql.put, "DateTimeItem", url, actions);
        yield put({
          type: FORM_API.DATETIME.PUT.SUCCESS,
          data: response.data.data.updateDateTimeItem
        });
        return yield call(complete, true);
      case TIME:
        var response = yield call(graphql.put, "TimeItem", url, actions);
        yield put({
          type: FORM_API.TIME.PUT.SUCCESS,
          data: response.data.data.updateTimeItem
        });
        return yield call(complete, true);
      case SCALE:
        var response = yield call(graphql.put, "ScaleItem", url, actions);
        yield put({
          type: FORM_API.SCALE.PUT.SUCCESS,
          data: response.data.data.updateScaleItem
        });
        return yield call(complete, true);
      case GRID:
        var response = yield call(graphql.put, "GridItem", url, actions);
        yield put({
          type: FORM_API.GRID.PUT.SUCCESS,
          data: response.data.data.updateGridItem
        });
        return yield call(complete, true);
      case CHECKBOX_GRID:
        var response = yield call(
          graphql.put,
          "CheckboxGridItem",
          url,
          actions
        );
        yield put({
          type: FORM_API.CHECKBOX_GRID.PUT.SUCCESS,
          data: response.data.data.updateCheckboxGridItem
        });
        return yield call(complete, true);
      case IMAGE:
        var response = yield call(graphql.put, "ImageItem", url, actions);
        yield put({
          type: FORM_API.IMAGE.PUT.SUCCESS,
          data: response.data.data.updateImageItem
        });
        return yield call(complete, true);
      case VIDEO:
        var response = yield call(graphql.put, "VideoItem", url, actions);
        yield put({
          type: FORM_API.VIDEO.PUT.SUCCESS,
          data: response.data.data.updateVideoItem
        });
        return yield call(complete, true);
      case DROPDOWN:
        var response = yield call(graphql.put, "DropdownItem", url, actions);
        yield put({
          type: FORM_API.DROPDOWN.PUT.SUCCESS,
          data: response.data.data.updateDropdownItem
        });
        return yield call(complete, true);
      case RESPONSE:
        var response = yield call(graphql.put, "FormResponse", url, actions);
        yield put({
          type: FORM_API.RESPONSE.PUT.SUCCESS,
          data: response.data.data.updateFormResponse
        });
        if (actions.obj) {
          if (actions.obj.isPublic) {
            yield call(callback, true);
            return yield put({
              type: REDIRECT_TO,
              redirectTo: "/form/do/done"
            });
          }
        }
        if (
          actions.item.status === "PASS" ||
          actions.item.status === "SUBMITTED" ||
          actions.item.status === "FAIL"
        ) {
          yield call(callback, true);
          return history.back();
        } else {
          return yield call(callback, true);
        }
      case MOVE:
        var response = yield call(
          graphql.graph,
          `mutation{
  moveFormItem(formId:"${actions.item.formId}",itemId:"${
            actions.item.itemId
          }",to:${actions.item.to}){
    ${actions.query.map(v => v + " ")}
}}`,
          {},
          url,
          actions
        );
        yield put({
          type: FORM_API.SELF.GET.SUCCESS,
          data: response.data.data.moveFormItem
        });
        return yield call(complete, true);
      case EQUATION:
        var response = yield call(graphql.put, "Evaluation", url, actions);
        yield put({
          type: FORM_API.EQUATION.PUT.SUCCESS,
          data: response.data.data.updateEvaluation
        });
        return yield call(complete, true);
      case CONDITION:
        yield call(loading);
        var response = yield call(graphql.put, "Condition", url, actions);
        yield put({
          type: FORM_API.CONDITION.PUT.SUCCESS,
          data: response.data.data.updateCondition
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* del(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(graphql.delete, "Form", url, actions);
        yield put({
          type: FORM_API.SELF.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/form"
        });
      case ITEM:
        var response = yield call(
          graphql.graph,
          `mutation{
  deleteItem(formId:"${actions.item.formId}",itemId:"${actions.item.itemId}")
}`,
          null,
          url,
          actions
        );
        yield put({
          type: FORM_API.ITEM.DEL.SUCCESS,
          data: actions.item.itemId
        });
        return yield call(complete, true);
      case CHECKBOX:
        yield call(loading);
        var response = yield call(graphql.put, "CheckboxItem", url, actions);
        yield put({
          type: FORM_API.CHECKBOX.PUT.SUCCESS,
          data: response.data.data.updateCheckboxItem
        });
        return yield call(complete, true);
      case MULTIPLE_CHOICE:
        yield call(loading);
        var response = yield call(
          graphql.put,
          "MultipleChoiceItem",
          url,
          actions
        );
        yield put({
          type: FORM_API.MULTIPLE_CHOICE.PUT.SUCCESS,
          data: response.data.data.updateMultipleChoiceItem
        });
        return yield call(complete, true);
      case CHECKBOX_GRID:
        yield call(loading);
        var response = yield call(
          graphql.put,
          "CheckboxGridItem",
          url,
          actions
        );
        yield put({
          type: FORM_API.CHECKBOX_GRID.PUT.SUCCESS,
          data: response.data.data.updateCheckboxGridItem
        });
        return yield call(complete, true);
      case GRID:
        yield call(loading);
        var response = yield call(graphql.put, "GridItem", url, actions);
        yield put({
          type: FORM_API.GRID.PUT.SUCCESS,
          data: response.data.data.updateGridItem
        });
        return yield call(complete, true);
      case DROPDOWN:
        yield call(loading);
        var response = yield call(graphql.put, "DropdownItem", url, actions);
        yield put({
          type: FORM_API.DROPDOWN.PUT.SUCCESS,
          data: response.data.data.updateDropdownItem
        });
        return yield call(complete, true);
      case SECTION_HEADER:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation {deleteFormItemBetween(startItemId: "${
            actions.obj.startItemId
          }", endItemId: "${actions.obj.endItemId}",formId:"${
            actions.obj.formId
          }"){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url
        );
        yield put({
          type: FORM_API.SELF.GET.SUCCESS,
          data: response.data.data.deleteFormItemBetween
        });
        return yield call(complete, true);
      case RESPONSE:
        yield call(loading);
        var response = yield call(graphql.delete, "FormResponse", url, actions);
        yield put({
          type: FORM_API.RESPONSE.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return history.back();
      case EQUATION:
        yield call(loading);
        var response = yield call(graphql.delete, "Evaluation", url, actions);
        yield put({
          type: FORM_API.EQUATION.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/form/edit?id=" + actions.obj.formId + "&menu=equation"
        });
      case CONDITION:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation{
  deleteCondition(conditionId:"${actions.obj.conditionId}",id:"${
            actions.obj.id
          }")
}`,
          {},
          url,
          actions
        );
        yield put({
          type: FORM_API.CONDITION.DEL.SUCCESS,
          data: actions.obj.conditionId
        });
        return yield call(complete, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(
          graphql.search,
          "forms",
          actions.item.skip,
          10,
          url,
          actions
        );
        return yield put({
          type: FORM_API.SELF.ALL.SUCCESS,
          data: response.data.data.forms
        });
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
export function* watchForm() {
  yield takeEvery(FORM_REQUEST, request);
}
