import {
  COMPONENT,
  GET,
  POST,
  PUT,
  DEL,
  ALL,
  GROUP_API,
  GROUP_REQUEST,
  SELF,
  REDIRECT_TO,
  SEARCH,
  FORM,
  CLONE,
  COMPARE,
  MEMBER,
  CLEAR
} from "../actions/actionType.js";
import { put, fork, call, take, takeEvery } from "redux-saga/effects";
import graphql from "../services/graphql";
import query from "query";
const url = "group";
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error.response.data.message
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield fork(clear, actions);
    default:
      null;
  }
}
function* clear(actions) {
  switch (actions.doc) {
    case SELF:
      return yield put({
        type: GROUP_API.SELF.CLEAR.SUCCESS
      });
    default:
      break;
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        const service = actions.obj
          ? actions.obj.root
            ? "rootGroups"
            : "groups"
          : "groups";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          actions
        );
        yield put({
          type: GROUP_API.SELF.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      default:
        null;
    }
  } catch (e) {
    return yield call(complete);
    console.log(e);
    //yield call(callback, false, e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `group(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: GROUP_API.SELF.GET.SUCCESS,
          data: response.data.data.group
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* post(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(graphql.post, "Group", url, actions);
        yield put({
          type: GROUP_API.SELF.POST.SUCCESS,
          data: response.data.data.insertGroup
        });
        yield call(callback, true);
        if (actions.item.parentGroupId === undefined) {
          return yield put({
            type: REDIRECT_TO,
            redirectTo: "/group"
          });
        } else {
          return yield put({
            type: REDIRECT_TO,
            redirectTo: "/group/edit/sub?id=" + response.data.data.insertGroup.id
          });
        }

      case MEMBER:
        yield call(loading);
        var response = yield call(graphql.post, "Member", url, actions);
        yield put({
          type: GROUP_API.MEMBER.POST.SUCCESS,
          data: response.data.data.insertMember
        });
        return yield call(callback, true);
      case FORM:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation{
  insertFormItem(formId:"${actions.obj.formId}",groupId:"${
            actions.obj.groupId
          }"){
    ${actions.query.map(v => v + " ")}
  }
}`,
          {},
          url
        );
        yield put({
          type: GROUP_API.FORM.POST.SUCCESS,
          data: response.data.data.insertFormItem
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(graphql.put, "Group", url, actions);
        yield put({
          type: GROUP_API.SELF.PUT.SUCCESS,
          data: response.data.data.updateGroup
        });
        return yield call(callback, true);
      case MEMBER:
        var response = yield call(graphql.put, "Member", url, actions);
        yield put({
          type: GROUP_API.MEMBER.PUT.SUCCESS,
          data: response.data.data.updateMember
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* del(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(graphql.delete, "Group", url, actions);
        yield put({
          type: GROUP_API.SELF.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: actions.item.parentGroupId
            ? "/group/edit?id=" + actions.item.parentGroupId
            : "/group"
        });
      case MEMBER:
        var response = yield call(
          graphql.graph,
          `mutation{deleteMember(id:"${actions.item.id}",userId:"${
            actions.item.userId
          }")}`,
          {},
          url
        );
        yield put({
          type: GROUP_API.MEMBER.DEL.SUCCESS,
          data: actions.item.userId
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
export function* watchGroup() {
  yield takeEvery(GROUP_REQUEST, request);
}
