import {
  COMPONENT,
  GET,
  POST,
  PUT,
  DEL,
  ALL,
  CLEAR,
  LOGBOOK_API,
  LOGBOOK_REQUEST,
  SELF,
  REDIRECT_TO,
  SEARCH,
  PROCEDURE,
  LOGBOOK_FORM,
  FILE,
  LOGBOOK_MANAGE,
  VALIDATE,
  COMMENT,
  DISEASE,
  INSTITUTE,
  DISEASE_UPLOAD,
  PROCEDURE_UPLOAD
} from "../actions/actionType.js";
import { put, fork, call, take, takeEvery } from "redux-saga/effects";
import graphql from "../services/graphql";
import query from "query";
const url = "logbook";
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error.response.data.message
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield fork(clear, actions);
    default:
      null;
  }
}
function* clear(actions) {
  switch (actions.doc) {
    case SELF:
      return yield put({
        type: LOGBOOK_API.SELF.CLEAR.SUCCESS
      });
    default:
      break;
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var service = "meLogbooks";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          actions
        );
        yield put({
          type: LOGBOOK_API.SELF.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case INSTITUTE:
        yield call(loading);
        var me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        var service = `meLogbooks`;
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          {
            ...actions,
            item: { specialtyId: { $oid: me.data.data.me.specialty.id } }
          }
        );
        yield put({
          type: LOGBOOK_API.INSTITUTE.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case PROCEDURE:
        yield call(loading);
        var me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        var service = actions.obj
          ? actions.obj.isGroup
            ? "procedureGroups"
            : "procedures"
          : "procedureGroups";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          {
            ...actions,
            item: { specialtyId: { $oid: me.data.data.me.specialty.id } }
          }
        );
        yield put({
          type: LOGBOOK_API.PROCEDURE.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case DISEASE:
        yield call(loading);
        var me = yield call(graphql.get, "me", "account", {
          query: query.auth.me
        });
        var service = actions.obj
          ? actions.obj.isGroup
            ? "diseaseGroups"
            : "diseases"
          : "diseaseGroups";
        var response = yield call(
          graphql.search,
          service,
          actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
          actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
          url,
          {
            ...actions,
            item: { specialtyId: { $oid: me.data.data.me.specialty.id } }
          }
        );
        yield put({
          type: LOGBOOK_API.DISEASE.ALL.SUCCESS,
          data: response.data.data[service]
        });
        if (response.data.data[service].length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case LOGBOOK_MANAGE:
        try {
          yield call(loading);
          var me = yield call(graphql.get, "me", "account", {
            query: query.auth.me
          });
          var service = "logbookManages";
          var response = yield call(
            graphql.search,
            service,
            actions.obj ? (actions.obj.skip ? actions.obj.skip : 0) : 0,
            actions.obj ? (actions.obj.limit ? actions.obj.limit : 10) : 10,
            url,
            {
              ...actions,
              item: { specialtyId: { $oid: me.data.data.me.specialty.id } }
            }
          );
          yield put({
            type: LOGBOOK_API.LOGBOOK_MANAGE.ALL.SUCCESS,
            data: response.data.data[service]
          });
          if (response.data.data[service].length == 0) {
            return yield call(outOfRange);
          } else {
            return yield call(complete);
          }
        } catch (e) {
          console.log(e);
        }
      default:
        null;
    }
  } catch (e) {
    return yield call(complete);
    console.log(e);
    //yield call(callback, false, e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `logbook(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: LOGBOOK_API.SELF.GET.SUCCESS,
          data: response.data.data.logbook
        });
        return yield call(complete);
      case LOGBOOK_FORM:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `logbookFormResponseByFormResponseId(formResponseId:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: LOGBOOK_API.LOGBOOK_FORM.GET.SUCCESS,
          data: response.data.data.logbookFormResponseByFormResponseId
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* post(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var procedures = actions.item.procedure;
        var diseases = actions.item.disease;
        delete actions.item["procedure"];
        delete actions.item["disease"];
        var response = yield call(graphql.post, "Logbook", url, actions);
        yield put({
          type: LOGBOOK_API.SELF.POST.SUCCESS,
          data: response.data.data.insertLogbook
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo:
            "/logbook/create?id=" + response.data.data.insertLogbook.id
        });
      case PROCEDURE:
        var response = yield call(
          graphql.graph,
          `mutation insertOrUpdateLogbookProcedures($data:[LogbookProcedureInput]){insertOrUpdateLogbookProcedures(data:$data${
            actions.obj ? "," + actions.obj : ""
          }){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url,
          actions
        );
        return yield put({
          type: LOGBOOK_API.PROCEDURE.POST.SUCCESS,
          data: response.data.data.insertOrUpdateLogbookProcedures
        });
      case DISEASE:
        var response = yield call(
          graphql.graph,
          `mutation insertOrUpdateLogbookDiseases($data:[LogbookDiseaseInput]){insertOrUpdateLogbookDiseases(data:$data${
            actions.obj ? "," + actions.obj : ""
          }){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url,
          actions
        );
        return yield put({
          type: LOGBOOK_API.DISEASE.POST.SUCCESS,
          data: response.data.data.insertOrUpdateLogbookDiseases
        });
      case LOGBOOK_FORM:
        yield call(loading);
        var forms = yield call(
          graphql.graph,
          `mutation insertFormResponse($data:FormResponseInput){insertFormResponse(data:$data,formId:"${
            actions.id
          }"){${actions.query.map(v => v + " ")}}}`,
          { data: actions.obj },
          "form"
        );
        var response = yield call(
          graphql.graph,
          `mutation{
  insertLogbookFormResponse(id:{formResponseId:"${
    forms.data.data.insertFormResponse.id
  }",logbookId:"${actions.item.logbookId}"}){
    id {
    logbookId
    formResponseId
  }
  formResponse{
    id,
    form{
      id
      title
    }
    requestUser{
      username
      firstNameTh
      lastNameTh
    }
    responseUser{
      username
      firstNameTh
      lastNameTh
    }
    score,
    updatedOn
  }
  }
}`,
          {},
          url,
          actions
        );
        yield put({
          type: LOGBOOK_API.LOGBOOK_FORM.POST.SUCCESS,
          data: response.data.data.insertLogbookFormResponse
        });
        return yield call(callback, true);
      case FILE:
        yield put({
          type: LOGBOOK_API.FILE.POST.SUCCESS,
          data: actions.item
        });
        return yield call(callback, true);
      case COMMENT:
        yield call(loading);
        var response = yield call(graphql.post, "LogbookComment", url, actions);
        return yield put({
          type: LOGBOOK_API.COMMENT.POST.SUCCESS,
          data: response.data.data.insertLogbookComment
        });
      case DISEASE_UPLOAD:
        var response = yield call(
          graphql.graph,
          `mutation insertDiseaseWithGroup($data: [DiseaseWithGroupInput]){
              insertDiseaseWithGroup(specialtyId:"5c2c79e870aa490001fd7ae6",datas:$data)
            }`,
          {
            data: actions.obj.data.map(v => ({
              diseaseGroupName: v.diseaseGroupName,
              diseaseCode: v.diseaseCode,
              diseaseName: v.diseaseName,
              minRequire: v.minRequire
            }))
          },
          url,
          actions
        );

        yield put({
          type: LOGBOOK_API.DISEASE_UPLOAD.POST.SUCCESS,
          data: response.data.data.insertDiseaseWithGroup
        });
        if (actions.obj) {
          return history.back();
        } else {
          return yield call(callback, true);
        }
      case PROCEDURE_UPLOAD:
        var response = yield call(
          graphql.graph,
          `mutation insertProcedureWithGroup($data: [ProcedureWithGroupInput]){
              insertProcedureWithGroup(specialtyId:"5c2c79e870aa490001fd7ae6",datas:$data)
              }`,
          {
            data: actions.obj.data.map(v => ({
              procedureGroupName: v.diseaseGroupName,
              procedureCode: v.diseaseCode,
              procedureName: v.diseaseName,
              minRequire: v.minRequire
            }))
          },
          url,
          actions
        );

        yield put({
          type: LOGBOOK_API.PROCEDURE_UPLOAD.POST.SUCCESS,
          data: response.data.data.insertProcedureWithGroup
        });
        if (actions.obj) {
          return history.back();
        } else {
          return yield call(callback, true);
        }

      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        var procedures = actions.item.procedure;
        var diseases = actions.item.disease;
        delete actions.item["procedure"];
        delete actions.item["disease"];
        var response = yield call(graphql.put, "Logbook", url, actions);
        yield put({
          type: LOGBOOK_API.SELF.PUT.SUCCESS,
          data: response.data.data.updateLogbook
        });
        if (procedures) {
          yield put({
            type: LOGBOOK_REQUEST,
            api: POST,
            doc: PROCEDURE,
            item: procedures.map(v => ({
              id: {
                logbookId: response.data.data.updateLogbook.id,
                procedureId: v
              }
            })),
            query: query.logbook.logbook_procedures,
            obj: `logbookId:"${response.data.data.updateLogbook.id}"`
          });
        }
        if (diseases) {
          yield put({
            type: LOGBOOK_REQUEST,
            api: POST,
            doc: DISEASE,
            item: diseases.map(v => ({
              id: {
                logbookId: response.data.data.updateLogbook.id,
                diseaseId: v
              }
            })),
            query: query.logbook.logbook_diseases,
            obj: `logbookId:"${response.data.data.updateLogbook.id}"`
          });
        }
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "BACK"
        });
      case VALIDATE:
        var response = yield call(
          graphql.graph,
          `mutation{
  logbookValidate(id:"${actions.id}",status:${actions.item.status},by:"${
            actions.item.by
          }"){
    ${actions.query.map(v => v + " ")}
  }
}`,
          {},
          url,
          actions
        );
        yield put({
          type: LOGBOOK_API.VALIDATE.PUT.SUCCESS,
          data: response.data.data.logbookValidate
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* del(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(graphql.delete, "Logbook", url, actions);
        yield put({
          type: LOGBOOK_API.SELF.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/logbook"
        });
      case LOGBOOK_FORM:
        var response = yield call(
          graphql.graph,
          `mutation {
    deleteLogbookFormResponse(id: {formResponseId: "${
      actions.id.formResponseId
    }", logbookId: "${actions.id.logbookId}"})
  }
  `,
          {},
          url,
          actions
        );
        yield put({
          type: LOGBOOK_API.LOGBOOK_FORM.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return history.back();
      case FILE:
        var response = yield call(
          graphql.del,
          `/logbook/${actions.id}/file/${actions.item.fileId}`
        );
        yield put({
          type: LOGBOOK_API.FILE.DEL.SUCCESS,
          data: actions.item.fileId
        });
        return yield call(callback, true);
      case COMMENT:
        var response = yield call(
          graphql.delete,
          "LogbookComment",
          url,
          actions
        );
        yield put({
          type: LOGBOOK_API.COMMENT.DEL.SUCCESS,
          data: actions.id
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
export function* watchLogbook() {
  yield takeEvery(LOGBOOK_REQUEST, request);
}
