import query from "query";
import { call, fork, put, takeEvery, throttle } from "redux-saga/effects";
import {
  ALL,
  ANNOTATE,
  CLEAR,
  COMPONENT,
  DEL,
  EXPORT_CSV,
  FILE,
  FOLLOWUP,
  FORM,
  FORM_REQUEST,
  GET,
  GROUP_REQUEST,
  HEADER,
  LATEST_FOLLOWUP,
  MEDIC_API,
  MEDIC_REQUEST,
  MEDIC_RT_REQUEST,
  PATIENT,
  POST,
  PUT,
  REDIRECT_TO,
  SEARCH,
  SELF,
  STAT,
  STAT_GROUP
} from "../actions/actionType.js";
import graphql from "../services/graphql";
const url = "medic";
const convert = item => {
  return JSON.stringify(item).replace(/\"([^(\")"]+)\":/g, "$1:");
};
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error.response.data.message
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield fork(clear, actions);
    default:
      null;
  }
}
function* rtRequest(actions) {
  switch (actions.api) {
    case PUT:
      return yield fork(rtChange, actions);
    default:
      null;
  }
}
function* clear(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        return yield put({
          type: MEDIC_API.SELF.CLEAR.SUCCESS,
          data: []
        });
      case FOLLOWUP:
        return yield put({
          type: MEDIC_API.FOLLOWUP.CLEAR.SUCCESS,
          data: {}
        });
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case PATIENT:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "patients",
          actions.obj.skip,
          10,
          url,
          actions,
          actions.sort
        );
        if (actions.obj.skip === 0) {
          yield put({
            type: MEDIC_API.PATIENT.CLEAR.SUCCESS
          });
        }
        yield put({
          type: MEDIC_API.PATIENT.ALL.SUCCESS,
          data: response.data.data.patients
        });
        if (response.data.data.patients.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case FOLLOWUP:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "followUps",
          actions.obj.skip,
          100,
          url,
          actions
        );
        yield put({
          type: MEDIC_API.FOLLOWUP.ALL.SUCCESS,
          data: response.data.data.followUps
        });
        if (response.data.data.followUps.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case HEADER:
        yield call(loading);
        var response = yield call(
          graphql.getRest,
          `medic/patients/import/headers`
        );
        yield put({
          type: MEDIC_API.HEADER.ALL.SUCCESS,
          data: response.data
        });
        return yield call(complete);
      case STAT_GROUP:
        yield call(loading);
        var response = yield call(
          graphql.getRest,
          `medic/group/${actions.id.id}/groupFormAnswerIdByRange
`,
          actions.item
        );
        yield put({
          type: MEDIC_API.STAT_GROUP.ALL.SUCCESS,
          data: JSON.stringify(response.data)
        });
        if (actions.id.isCSV) {
          yield put({
            type: FORM_REQUEST,
            api: ALL,
            doc: EXPORT_CSV,
            item: response.data,
            id: actions.id.formId
          });
        } else {
          yield put({
            type: FORM_REQUEST,
            api: ALL,
            doc: STAT_GROUP,
            item: convert(response.data),
            id: actions.id.formId,
            query: query.form.stats
          });
        }
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    return yield call(complete);
    console.log(e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case PATIENT:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `patient(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: MEDIC_API.PATIENT.GET.SUCCESS,
          data: response.data.data.patient
        });
        yield put({
          type: GROUP_REQUEST,
          api: GET,
          doc: SELF,
          id: response.data.data.patient.group.id,
          query: query.group.data
        });
        return yield call(complete);
      case FOLLOWUP:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `followUp(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: MEDIC_API.FOLLOWUP.GET.SUCCESS,
          data: response.data.data.followUp
        });
        return yield call(complete);
      case LATEST_FOLLOWUP:
        yield call(loading);
        var response = yield call(
          graphql.filter,
          `todayLatestFollowUp`,
          url,
          actions
        );
        var fu = response.data.data
          ? response.data.data.todayLatestFollowUp
          : null;
        yield put({
          type: MEDIC_API.FOLLOWUP.GET.SUCCESS,
          data: fu
        });
        return yield call(complete);
      case STAT:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `query formResponses($data: Object) {
  patient(id: "${actions.id}") {
    formResponses(query: $data,size:1000) {
      id
      formId
      evaluations {
        id
        eval2
        name
      }
      updatedOn
      followUp{
        followUpDate
      }
    }
  }
}
`,
          actions.item,
          url,
          actions
        );
        yield put({
          type: MEDIC_API.STAT.GET.SUCCESS,
          data: response.data.data.patient,
          compare: actions.obj ? true : false
        });
        return yield call(complete);
      case STAT_GROUP:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `query groupFormResponses($data: Object){
  groupFormResponses(groupId:"${actions.id}",query: $data,size:10000){
  	id
    followUp{
      followUpDate
    }
    updatedOn
    evaluations{
      id
      eval2
      name
    }
  }
}
        `,
          actions.item,
          url,
          actions
        );
        yield put({
          type: MEDIC_API.STAT_GROUP.GET.SUCCESS,
          data: response.data.data.groupFormResponses
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    //yield call(callback, false, e);
  }
}
function* post(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case PATIENT:
        yield call(loading);
        var response = yield call(graphql.post, "Patient", url, actions);
        yield put({
          type: MEDIC_API.PATIENT.POST.SUCCESS,
          data: response.data.data.insertPatient
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo:
            "/followup?patientId=" + response.data.data.insertPatient.id
        });
      case FOLLOWUP:
        yield call(loading);
        var response = yield call(graphql.post, "FollowUp", url, actions);
        yield put({
          type: MEDIC_API.FOLLOWUP.POST.SUCCESS,
          data: response.data.data.insertFollowUp
        });
        return yield call(callback, true);
      case FORM:
        var response = yield call(
          graphql.graph,
          `mutation{insertFollowUpFormResponse(id:"${
            actions.item.id
          }",formResponseId:"${actions.item.responseId}"){
            ${actions.query.map(v => v + " ")}
          }}`,
          {},
          url
        );
        yield put({
          type: MEDIC_API.SELF.GET.SUCCESS,
          data: response.data.data.insertFollowUpFormResponse
        });
        return yield call(complete);
      case FILE:
        yield put({
          type: MEDIC_API.FILE.POST.SUCCESS,
          data: actions.item
        });
        return yield call(complete);
      case ANNOTATE:
        yield call(loading);
        var response = yield call(
          graphql.upload,
          `/file/files/${actions.id.id}`,
          actions.item
        );
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/followup?patientId=" + actions.id.patientId
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  try {
    switch (actions.doc) {
      case PATIENT:
        yield call(loading);
        var response = yield call(graphql.put, "Patient", url, actions);
        yield put({
          type: MEDIC_API.PATIENT.PUT.SUCCESS,
          data: response.data.data.updatePatient
        });
        return yield call(callback, true);
      case FOLLOWUP:
        var response = yield call(graphql.put, "FollowUp", url, actions);
        yield put({
          type: MEDIC_API.FOLLOWUP.PUT.SUCCESS,
          data: response.data.data.updateFollowUp
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* del(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case FORM:
        var response = yield call(
          graphql.graph,
          `mutation{
  deleteFollowUpFormResponse(id:"${actions.id}",formResponseId:"${
            actions.item.formResponseId
          }")
}`,
          {},
          url
        );
        yield put({
          type: MEDIC_API.FORM.DEL.SUCCESS,
          data: actions.item.formResponseId
        });
        return yield call(callback, true);
      case FILE:
        var response = yield call(
          graphql.del,
          `/medic/followUp/${actions.id}/file/${actions.item.fileId}`
        );
        yield put({
          type: MEDIC_API.FILE.DEL.SUCCESS,
          data: actions.item.fileId
        });
        return yield call(callback, true);
      case FOLLOWUP:
        var response = yield call(graphql.delete, "FollowUp", url, actions);
        yield put({
          type: MEDIC_API.FOLLOWUP.DEL.SUCCESS,
          data: actions.id
        });
        return yield call(callback, true);
      case PATIENT:
        var response = yield call(graphql.delete, "Patient", url, actions);
        yield put({
          type: MEDIC_API.PATIENT.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/"
        });

      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}

export function* watchMedic() {
  yield throttle(400, MEDIC_RT_REQUEST, request);
  yield takeEvery(MEDIC_REQUEST, request);
}
