import {
  COMPONENT,
  GET,
  POST,
  PUT,
  DEL,
  ALL,
  RESEARCH_REQUEST,
  RESEARCH_API,
  SELF,
  REDIRECT_TO,
  SEARCH,
  FORM,
  CLONE,
  COMPARE,
  CLEAR,
  EVALUATION,
  FILE,
  GETID,
  COMMENT,
  FORM_EVALUATION,
  MANU_SCRIPT,
  CLONE_MANU,
  MANAGE,
  ACTIVITY,
  MANAGE_ACTIVITY
} from "../actions/actionType.js";
import { put, fork, call, take, takeEvery, throttle } from "redux-saga/effects";
import graphql from "../services/graphql";
import query from "query";
const url = "research";
import auth from "../services/auth";
import roles from "../roles";
const convert = item => {
  return JSON.stringify(item).replace(/\"([^(\")"]+)\":/g, "$1:");
};
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error.response.data.message
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield put({
        type: RESEARCH_API.CLEAR.ALL.SUCCESS,
        data: []
      });
    default:
      null;
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        const me = yield call(auth.me);
        var service = "researches";
        if (me.data.roles.length > 0) {
          service = roles.permission(me.data.roles, ["ADMIN"])
            ? "researches"
            : roles.permission(me.data.roles, ["STUDENT"])
            ? "researchesMe"
            : roles.permission(me.data.roles, ["TEACHER"])
            ? "researchesCommittee"
            : "researches";
        }
        var currentStatus = actions.obj.currentStatus;
        var q = currentStatus
          ? `${actions.obj.skip},currentStatus:${currentStatus}`
          : `${actions.obj.skip}`;
        q += actions.obj["instituteId"]
          ? ",instituteId:" + `"${actions.obj["instituteId"]}"`
          : "";
        var response = yield call(graphql.search, service, q, 10, url, actions);
        if (actions.item.skip == 0) {
          yield put({
            type: RESEARCH_API.CLEAR.ALL.SUCCESS,
            data: []
          });
          yield call(renew);
        }
        var reseaches = response.data.data[service];
        yield put({
          type: RESEARCH_API.SELF.ALL.SUCCESS,
          data: reseaches
        });
        if (reseaches.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case EVALUATION:
        var response = yield call(
          graphql.filter,
          "evaluationScoreRespones",
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.EVALUATION.ALL.SUCCESS,
          data: response.data.data.evaluationScoreRespones
        });
        return yield call(complete);
      case FORM_EVALUATION:
        var response = yield call(
          graphql.get,
          "evaluationScores",
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.FORM_EVALUATION.ALL.SUCCESS,
          data: response.data.data.evaluationScores
        });
        return yield call(complete);
      case MANAGE:
        var response = yield call(
          graphql.get,
          `manageResearches(size:1000,sortBy:"createdOn")`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.MANAGE.ALL.SUCCESS,
          data: response.data.data.manageResearches
        });
        return yield call(complete);
      case ACTIVITY:
        var response = yield call(
          graphql.get,
          "activities(size:1000)",
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.ACTIVITY.ALL.SUCCESS,
          data: response.data.data.activities
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(
          graphql.get,
          `research(id:"${actions.id}")`,
          url,
          {
            ...actions,
            query: [
              ...actions.query,
              `manageResearch{
      id
              name
              yearEnrolled
              start
              end
              activities(sortBy: "expectedDate", sort: "ASC"){
                id
                expiredDate
                expectedDate
                nameTh
                researchActivity(researchId:"${actions.id}"){
                  submitted
                  id{
                    activityId
                  }
                  verifier{
                    username
                    firstNameEn
                    lastNameEn
                  }
                  verifiedDate
                  files{
                    file{
                      id
                      name
                    }
                    submitted
                  }
                }
              }
            }`
            ]
          }
        );
        yield put({
          type: RESEARCH_REQUEST,
          api: GET,
          doc: FORM,
          item: null,
          id:
            response.data.data.research.researchForms.filter(
              v => v.status === response.data.data.research.currentStatus
            ).length > 0
              ? response.data.data.research.researchForms.filter(
                  v => v.status === response.data.data.research.currentStatus
                )[0].id
              : null,
          query: query.research.form
        });
        if (response.data.data.research.manuScriptForms.length > 0) {
          yield put({
            type: RESEARCH_REQUEST,
            api: GET,
            doc: MANU_SCRIPT,
            item: null,
            id: response.data.data.research.manuScriptForms[0].id,
            query: query.research.manu_script
          });
        }
        yield put({
          type: RESEARCH_REQUEST,
          api: GET,
          doc: COMPARE,
          item: null,
          id:
            response.data.data.research.researchForms.filter(
              v => v.status === response.data.data.research.currentStatus
            ).length > 0
              ? response.data.data.research.researchForms.filter(
                  v => v.status === response.data.data.research.currentStatus
                )[0].id
              : null,
          query: query.research.form
        });
        yield put({
          type: RESEARCH_API.SELF.GET.SUCCESS,
          data: response.data.data.research
        });
        return yield call(complete);
      case FORM:
        var response = yield call(
          graphql.get,
          `researchForm(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.FORM.GET.SUCCESS,
          data: response.data.data.researchForm
        });
        return yield call(complete);
      case MANU_SCRIPT:
        var response = yield call(
          graphql.get,
          `manuScriptForm(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.MANU_SCRIPT.GET.SUCCESS,
          data: response.data.data.manuScriptForm
        });
        return yield call(complete);
      case COMPARE:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `researchForm(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.COMPARE.GET.SUCCESS,
          data: response.data.data.researchForm
        });
        return yield call(complete);
      case EVALUATION:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `evaluationScoreResponse(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.EVALUATION.GET.SUCCESS,
          data: response.data.data
            ? response.data.data.evaluationScoreResponse
            : null
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* post(actions) {
  try {
    switch (actions.doc) {
      case SELF:
        yield call(loading);
        var response = yield call(graphql.post, "Research", url, actions);
        yield put({
          type: RESEARCH_API.SELF.POST.SUCCESS,
          data: response.data.data.insertResearch
        });
        const presetText = `<ol>
          <li>
            <br data-mce-bogus="1">
          </li>
          <li>
            <br data-mce-bogus="2">
          </li>
          <li>
            <br data-mce-bogus="3">
          </li>
          <li>
            <br data-mce-bogus="4">
          </li>
        </ol>`;
        yield put({
          type: RESEARCH_REQUEST,
          api: POST,
          doc: FORM,
          item: {
            status: "DRAFT",
            primaryObjective: presetText,
            secondaryObject: presetText,
            generalizability: presetText,
            inclusionCriteria: presetText,
            exclusionCriteria: presetText,
            operationDefinition: presetText,
            reference: presetText
          },
          id: "",
          query: query.research.forms,
          obj: `researchId:"${response.data.data.insertResearch.id}"`
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo:
            "/research/edit?id=" + response.data.data.insertResearch.id
        });
      case FORM:
        yield call(loading);
        var response = yield call(graphql.post, "ResearchForm", url, actions);
        yield put({
          type: RESEARCH_API.FORM.POST.SUCCESS,
          data: response.data.data.insertResearchForm
        });
        return yield call(callback, true);
      case COMMENT:
        var response = yield call(
          graphql.post,
          "ResearchComment",
          url,
          actions,
          actions.obj
        );
        return yield put({
          type: RESEARCH_API.COMMENT.POST.SUCCESS,
          data: response.data.data.insertResearchComment
        });
      case EVALUATION:
        var response = yield call(
          graphql.graph,
          `mutation{insertEvaluationScoreResponse(userId:"${
            actions.item.userId
          }",researchId:"${actions.item.researchId}"){
            ${actions.query.map(v => v + " ")}
          }}`,
          {},
          url
        );
        yield put({
          type: RESEARCH_API.EVALUATION.POST.SUCCESS,
          data: response.data.data.insertEvaluationScoreResponse
        });
        var res_evaluations = yield call(graphql.get, "evaluationScores", url, {
          ...actions,
          query: query.research.form_evaluations
        });
        var evaluations = res_evaluations.data.data.evaluationScores;
        for (var v in evaluations) {
          var res_form_evaluations = yield call(
            graphql.graph,
            `mutation{insertEvaluationScoreAnswer(score:null,evaluationScoreId:"${
              evaluations[v].id
            }",evaluationScoreResponseId:"${
              response.data.data.insertEvaluationScoreResponse.id
            }"){
            ${[`id`].map(v => v + " ")}
          }}`,
            {},
            url
          );
        }
        yield put({
          type: REDIRECT_TO,
          redirectTo:
            "/research/edit/components/score/form?id=" +
            response.data.data.insertEvaluationScoreResponse.id
        });
        return yield call(callback, true);
      case FORM_EVALUATION:
        var response = yield call(
          graphql.graph,
          `mutation{insertEvaluationScoreAnswer(score:"${
            actions.item.score
          }",evaluationScoreId:"${
            actions.item.evaluationScoreId
          }",evaluationScoreResponseId:"${
            actions.item.evaluationScoreResponseId
          }"){
          ${actions.query.map(v => v + " ")}
        }}`,
          {},
          url
        );
        yield put({
          type: RESEARCH_API.FORM_EVALUATION.POST.SUCCESS,
          data: response.data.data.insertEvaluationScoreAnswer
        });
        return yield call(complete);
      case MANU_SCRIPT:
        yield call(loading);
        var response = yield call(graphql.post, "ManuScriptForm", url, actions);
        yield put({
          type: RESEARCH_API.MANU_SCRIPT.POST.SUCCESS,
          data: response.data.data.insertManuScriptForm
        });
        return yield call(callback, true);
      case MANAGE:
        yield call(loading);
        var response = yield call(graphql.post, "ManageResearch", url, actions);
        yield put({
          type: RESEARCH_API.MANAGE.POST.SUCCESS,
          data: response.data.data.insertManageResearch
        });
        return yield call(callback, true);
      case ACTIVITY:
        yield call(loading);
        var response = yield call(graphql.post, "Activity", url, actions);
        yield put({
          type: RESEARCH_API.ACTIVITY.POST.SUCCESS,
          data: response.data.data.insertActivity
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(graphql.put, "Research", url, {
          ...actions,
          query: [
            ...actions.query,
            `manageResearch{
    id
    name
    yearEnrolled
    activities{
      id
      expiredDate
      expectedDate
      nameTh
      researchActivity(researchId:"${actions.id}"){
        submitted
        id{
          activityId
        }
        verifier{
          username
          firstNameEn
          lastNameEn
        }
        verifiedDate
        files{
          file{
            id
            name
          }
          submitted
        }
      }
    }
  }`
          ]
        });
        yield put({
          type: RESEARCH_API.SELF.PUT.SUCCESS,
          data: response.data.data.updateResearch
        });
        return yield call(callback, true);
      case MANU_SCRIPT:
        var response = yield call(graphql.put, "ManuScriptForm", url, actions);
        yield put({
          type: RESEARCH_API.MANU_SCRIPT.PUT.SUCCESS,
          data: response.data.data.updateManuScriptForm
        });
        return yield call(callback, true);
      case FORM:
        var response = yield call(graphql.put, "ResearchForm", url, actions);
        yield put({
          type: RESEARCH_API.FORM.GET.SUCCESS,
          data: response.data.data.updateResearchForm
        });
        return yield call(callback, true);
      case CLONE:
        var response = yield call(
          graphql.clone,
          `cloneResearchForm(id:"${actions.id}",status:${actions.item.status})`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.FORM.PUT.SUCCESS,
          data: response.data.data.cloneResearchForm
        });
        yield put({
          type: RESEARCH_REQUEST,
          api: GET,
          doc: SELF,
          item: null,
          id: actions.item.id,
          query: query.research.data
        });
        return yield call(callback, true);
      case CLONE_MANU:
        var response = yield call(
          graphql.clone,
          `cloneManuScriptForm(id:"${actions.id}",status:${
            actions.item.status
          })`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.MANU_SCRIPT.PUT.SUCCESS,
          data: response.data.data.cloneManuScriptForm
        });
        yield put({
          type: RESEARCH_REQUEST,
          api: GET,
          doc: SELF,
          item: null,
          id: actions.item.id,
          query: query.research.manu_script
        });
        return yield call(callback, true);
      case EVALUATION:
        var response = yield call(
          graphql.custom,
          "updateResearchEvaluation",
          `isPass:${actions.item.isPass},id:"${actions.id}"`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.SELF.PUT.SUCCESS,
          data: response.data.data.updateResearchEvaluation
        });
        return yield call(callback, true);
      case FORM_EVALUATION:
        yield call(loading);
        var response = yield call(
          graphql.put,
          `EvaluationScoreAnswer`,
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.FORM_EVALUATION.PUT.SUCCESS,
          data: response.data.data.updateEvaluationScoreAnswer
        });
        return yield call(complete);
      case MANAGE:
        yield call(loading);
        var response = yield call(graphql.put, `ManageResearch`, url, actions);
        yield put({
          type: RESEARCH_API.MANAGE.PUT.SUCCESS,
          data: response.data.data.updateManageResearch
        });
        return yield call(complete);
      case ACTIVITY:
        yield call(loading);
        var response = yield call(graphql.put, `Activity`, url, actions);
        yield put({
          type: RESEARCH_API.ACTIVITY.PUT.SUCCESS,
          data: response.data.data.updateActivity
        });
        return yield call(complete);
      case MANAGE_ACTIVITY:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation updateResearchActivity($data:ResearchActivityInput){updateResearchActivity(data:$data,id:${convert(
            actions.id
          )}){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.MANAGE_ACTIVITY.PUT.SUCCESS,
          data: response.data.data.updateResearchActivity
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* del(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case SELF:
        var response = yield call(graphql.delete, "Research", url, actions);
        yield put({
          type: RESEARCH_API.SELF.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/research"
        });
      case FILE:
        if (actions.item.activityId) {
          var response = yield call(
            graphql.del,
            `/research/${actions.item.id}/activity/${
              actions.item.activityId
            }/file/${actions.id}`
          );
          yield put({
            type: RESEARCH_REQUEST,
            doc: SELF,
            api: GET,
            id: actions.item.id,
            query: actions.query
          });
        } else {
          var response = yield call(
            graphql.del,
            `/research/${actions.item.id}/field/${
              actions.item.fieldName
            }/file/${actions.id}`
          );
          yield put({
            type: RESEARCH_REQUEST,
            doc: FORM,
            api: GET,
            id: actions.item.id,
            query: actions.query
          });
        }
        return yield call(callback, true);
      case EVALUATION:
        var response = yield call(
          graphql.delete,
          "EvaluationScoreResponse",
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.EVALUATION.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        // return yield put({
        //   type: REDIRECT_TO,
        //   redirectTo: "/research/edit?id=" + actions.item.researchId
        // });
        return history.back();
      case MANAGE:
        var response = yield call(
          graphql.delete,
          "ManageResearch",
          url,
          actions
        );
        yield put({
          type: RESEARCH_API.MANAGE.DEL.SUCCESS,
          data: actions.id
        });
        return yield call(callback, true);
      case ACTIVITY:
        var response = yield call(graphql.delete, "Activity", url, actions);
        yield put({
          type: RESEARCH_API.ACTIVITY.DEL.SUCCESS,
          data: actions.id
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
export function* watchResearch() {
  yield takeEvery(RESEARCH_REQUEST, request);
}
