import {
  COMPONENT,
  GET,
  POST,
  PUT,
  DEL,
  ALL,
  WFME_API,
  WFME_REQUEST,
  SELF,
  REDIRECT_TO,
  SEARCH,
  FORM,
  CLONE,
  COMPARE,
  KPI,
  COURSE,
  COURSE_OFFER,
  ANSWER,
  RESPONSE,
  FORM_REQUEST,
  FILE,
  OUTCOME,
  DEPARTMENT,
  CURRICULUM,
  FILE_NAME,
  CLEAR,
  BUNDLE,
  ACADYEAR,
  OUTCOME_FORM,
  CUCAS,
  CUCAS_USER,
  CUCAS_INSTRUCTOR
} from "../actions/actionType.js";
import { put, fork, call, take, takeEvery } from "redux-saga/effects";
import graphql from "../services/graphql";
import query from "query";
const url = "wfme";
function* callback(isSuccess, error) {
  isSuccess
    ? yield put({
        type: COMPONENT.CALLBACK_SUCCESS
      })
    : yield put({
        type: COMPONENT.CALLBACK_FAIL,
        data: error ? error.response.data.message : "Error"
      });
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* loading() {
  yield put({
    type: COMPONENT.CALLBACK_LOADING
  });
}
function* complete() {
  yield put({
    type: COMPONENT.SUCCESS
  });
}
function* outOfRange() {
  yield put({
    type: COMPONENT.CALLBACK_OUT
  });
}
function* renew() {
  yield put({
    type: COMPONENT.CALLBACK_NEW
  });
}
function* request(actions) {
  switch (actions.api) {
    case GET:
      return yield fork(get, actions);
    case POST:
      return yield fork(post, actions);
    case PUT:
      return yield fork(change, actions);
    case DEL:
      return yield fork(del, actions);
    case ALL:
      return yield fork(getAll, actions);
    case SEARCH:
      return yield fork(search, actions);
    case CLEAR:
      return yield put({
        type: WFME_API.SELF.CLEAR.SUCCESS
      });
    default:
      null;
  }
}
function* getAll(actions) {
  try {
    switch (actions.doc) {
      case CUCAS:
        yield call(loading);
        var response = yield call(
          graphql.getRest,
          `/wfme/cucas/course/search`,
          actions.item
        );
        yield put({
          type: WFME_API.CUCAS.ALL.SUCCESS,
          data: response.data
        });
        return yield call(complete);
      case CUCAS_USER:
        yield call(loading);
        var response = yield call(
          graphql.getRest,
          `/wfme/cucas/instructor/search`,
          actions.item
        );
        yield put({
          type: WFME_API.CUCAS_USER.ALL.SUCCESS,
          data: response.data
        });
        return yield call(complete);
      case FILE_NAME:
        yield call(loading);
        var response = yield call(
          graphql.get,
          "kpiFileNameGroups",
          url,
          actions
        );
        return yield put({
          type: WFME_API.FILE_NAME.ALL.SUCCESS,
          data: response.data.data.kpiFileNameGroups
        });
      case CURRICULUM:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "curriculums",
          actions.obj.skip,
          actions.obj.limit ? actions.obj.limit : 20,
          url,
          actions
        );
        yield put({
          type: WFME_API.CURRICULUM.ALL.SUCCESS,
          data: response.data.data.curriculums
        });
        if (response.data.data.curriculums.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case ACADYEAR:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `{
  academicYears
}`,
          {},
          url,
          actions
        );
        yield put({
          type: WFME_API.ACADYEAR.ALL.SUCCESS,
          data: response.data.data.academicYears
        });
        return yield call(complete);
      case KPI:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "kpis",
          actions.obj.skip,
          actions.obj.limit ? actions.obj.limit : 20,
          url,
          actions
        );
        yield put({
          type: WFME_API.KPI.ALL.SUCCESS,
          data: response.data.data.kpis
        });
        if (response.data.data.kpis.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case COURSE:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "courses",
          actions.obj.skip,
          20,
          url,
          actions
        );
        yield put({
          type: WFME_API.COURSE.ALL.SUCCESS,
          data: response.data.data.courses
        });
        if (response.data.data.courses.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case COURSE_OFFER:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "courseOffers",
          actions.obj.skip,
          20,

          url,
          actions
        );
        yield put({
          type: WFME_API.COURSE_OFFER.ALL.SUCCESS,
          data: response.data.data.courseOffers
        });
        if (response.data.data.courseOffers.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case ANSWER:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "courseOfferKpiAnswers",
          actions.obj.skip,
          100,
          url,
          actions
        );
        yield put({
          type: WFME_API.ANSWER.ALL.SUCCESS,
          data: response.data.data.courseOfferKpiAnswers
        });
        if (response.data.data.courseOfferKpiAnswers.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case OUTCOME:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "outcomes",
          0,
          100,
          url,
          actions
        );
        yield put({
          type: WFME_API.OUTCOME.ALL.SUCCESS,
          data: response.data.data.outcomes
        });
        if (response.data.data.outcomes.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case DEPARTMENT:
        yield call(loading);
        var response = yield call(
          graphql.search,
          "departments",
          0,
          100,
          url,
          actions
        );
        yield put({
          type: WFME_API.DEPARTMENT.ALL.SUCCESS,
          data: response.data.data.departments
        });
        if (response.data.data.departments.length == 0) {
          return yield call(outOfRange);
        } else {
          return yield call(complete);
        }
      case FILE:
        yield call(loading);
        var response = yield call(
          graphql.getRest,
          "file/directories/files",
          {}
        );
        yield put({
          type: WFME_API.FILE.ALL.SUCCESS,
          data: response.data
        });
        return yield call(complete);
      case OUTCOME_FORM:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `outcomeRelatedQAsByCourseOfferAndKpi(courseOfferId:"${
            actions.id.offerId
          }",kpiId:"${actions.id.kpiId}")`,
          url,
          actions
        );
        yield put({
          type: WFME_API.OUTCOME_FORM.ALL.SUCCESS,
          data: response.data.data.outcomeRelatedQAsByCourseOfferAndKpi
        });
        return yield call(complete);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* get(actions) {
  try {
    switch (actions.doc) {
      case KPI:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `kpi(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: WFME_API.KPI.GET.SUCCESS,
          data: response.data.data.kpi
        });
        return yield call(complete);
      case COURSE:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `course(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: WFME_API.COURSE.GET.SUCCESS,
          data: response.data.data.course
        });
        return yield call(complete);
      case COURSE_OFFER:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `courseOffer(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: WFME_API.COURSE_OFFER.GET.SUCCESS,
          data: response.data.data.courseOffer
        });
        return yield call(complete);
      case ANSWER:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `courseOfferKpiAnswer(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: WFME_API.ANSWER.GET.SUCCESS,
          data: response.data.data.courseOfferKpiAnswer
        });
        return yield call(complete);
      case RESPONSE:
        yield call(loading);
        var response = yield call(
          graphql.get,
          `courseOffer(id:"${actions.id}")`,
          url,
          actions
        );
        yield put({
          type: WFME_API.COURSE_OFFER.GET.SUCCESS,
          data: response.data.data.courseOffer
        });
        var form =
          response.data.data.courseOffer.kpiAnswers.filter(
            v => (v.kpi ? v.kpi.id : "") === actions.obj.kpiId
          ).length > 0
            ? response.data.data.courseOffer.kpiAnswers.filter(
                v => (v.kpi ? v.kpi.id : "") === actions.obj.kpiId
              )[0].formResponse
            : {};
        if (form) {
          yield put({
            type: FORM_REQUEST,
            api: GET,
            doc: SELF,
            item: {},
            id: form.formId,
            query: query.form.data
          });
          return yield put({
            type: FORM_REQUEST,
            api: GET,
            doc: RESPONSE,
            item: {},
            id: form.id,
            query: query.form.response
          });
        }
        return;

      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* post(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case CUCAS:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation{
  insertOrUpdateCoursesCucas(id:"${actions.id}",cucasCourseIds:${JSON.stringify(actions.item)}){
    id
  }
}`,
          {},
          url,
          actions
        );
        yield call(callback, true);
        return history.back();
      case CUCAS_INSTRUCTOR:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation{
            insertOrUpdateCourseOfferInstructorCucas(id:"${actions.id}",cucasInstructorId:"${actions.item.instructorid}",roles:${actions.item.roles}){
              userId
            }
          }`,
          {},
          url,
          actions
        );
        return yield call(callback, true);
      case KPI:
        yield call(loading);
        var response = yield call(graphql.post, "Kpi", url, actions);
        yield put({
          type: WFME_API.KPI.POST.SUCCESS,
          data: response.data.data.insertKpi
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/kpi"
        });
      case COURSE:
        yield call(loading);
        var response = yield call(graphql.post, "Course", url, actions);
        yield put({
          type: WFME_API.COURSE.POST.SUCCESS,
          data: response.data.data.insertCourse
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/course"
        });
      case COURSE_OFFER:
        yield call(loading);
        try {
          var response = yield call(
            graphql.graph,
            `mutation{
  openCourseOffers(academicYear:${
    actions.item.academicYear
  },courseIds:${JSON.stringify(actions.id)}){
    ${actions.query.map(v => v + " ")}
  }
}`,
            {},
            url,
            actions
          );
          if (response.data.data.openCourseOffers) {
            yield put({
              type: WFME_API.COURSE_OFFER.POST.SUCCESS,
              data:
                response.data.data.openCourseOffers.length > 0
                  ? response.data.data.openCourseOffers[0]
                  : {}
            });
            yield call(callback, true);
            return yield put({
              type: REDIRECT_TO,
              redirectTo:
                "/course/info?id=" +
                (response.data.data.openCourseOffers.length > 0
                  ? response.data.data.openCourseOffers[0].course.id
                  : "")
            });
          } else {
            return yield call(callback, false);
          }
        } catch (e) {
          console.log(e);
        }
      case ANSWER:
        yield call(loading);
        var response = yield call(
          graphql.post,
          "CourseOfferKpiAnswer",
          url,
          actions
        );
        yield put({
          type: WFME_API.ANSWER.POST.SUCCESS,
          data: response.data.data.insertCourseOfferKpiAnswer
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo:
            "/course/info/offer/evidence?id=" +
            response.data.data.insertCourseOfferKpiAnswer.id
        });
      case FILE:
        yield put({
          type: WFME_API.FILE.POST.SUCCESS,
          data: actions.item
        });
        return yield call(callback, true);
      case RESPONSE:
        var response = yield call(
          graphql.graph,
          `mutation{generateKpiFormResponses(kpiIds:${JSON.stringify(
            actions.item
          )},id:"${actions.id}"){
            ${actions.query.map(v => v + " ")}
  }}`,
          {},
          url,
          actions
        );
        yield put({
          type: WFME_API.RESPONSE.POST.SUCCESS,
          data: response.data.data.generateKpiFormResponses
        });
        return yield put({
          type: REDIRECT_TO,
          redirectTo:
            "/course/info/offer/KPIevidence?id=" +
            actions.id +
            "&kpiId=" +
            actions.item[0]
        });
      case BUNDLE:
        yield call(loading);
        var response = yield call(
          graphql.graph,
          `mutation{
  makeBundle(ids:${JSON.stringify(actions.item)}){
    ${actions.query.map(v => v + " ")}
  }
}`,
          {},
          url,
          actions
        );
        yield put({
          type: WFME_API.BUNDLE.POST.SUCCESS,
          data: response.data.data.makeBundle
        });
        yield call(callback, true);
        return history.back();
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* change(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case KPI:
        var response = yield call(graphql.put, "Kpi", url, actions);
        yield put({
          type: WFME_API.KPI.PUT.SUCCESS,
          data: response.data.data.updateKpi
        });
        return yield call(callback, true);
      case COURSE:
        var response = yield call(graphql.put, "Course", url, actions);
        yield put({
          type: WFME_API.COURSE.PUT.SUCCESS,
          data: response.data.data.updateCourse
        });
        return yield call(callback, true);
      case COURSE_OFFER:
        var response = yield call(graphql.put, "CourseOffer", url, actions);
        yield put({
          type: WFME_API.COURSE_OFFER.PUT.SUCCESS,
          data: response.data.data.updateCourseOffer
        });
        return yield call(callback, true);
      case ANSWER:
        var response = yield call(
          graphql.graph,
          `mutation insertOrUpdateCourseOfferKpiAnswer($data:CourseOfferKpiAnswerInput){insertOrUpdateCourseOfferKpiAnswer(data:$data,id:"${
            actions.id
          }"){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url,
          actions
        );
        yield put({
          type: WFME_API.ANSWER.PUT.SUCCESS,
          data: response.data.data.insertOrUpdateCourseOfferKpiAnswer
        });
        return history.back();
      case OUTCOME:
        var response = yield call(
          graphql.graph,
          `mutation insertOrUpdateCourseOfferOutcomeAnswer($data:CourseOfferOutcomeAnswerInput){insertOrUpdateCourseOfferOutcomeAnswer(data:$data,id: "${
            actions.id
          }"){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url,
          actions
        );
        yield put({
          type: WFME_API.OUTCOME.PUT.SUCCESS,
          data: response.data.data.insertOrUpdateCourseOfferOutcomeAnswer
        });
        return yield call(complete, true);
      case OUTCOME_FORM:
        var response = yield call(
          graphql.graph,
          `mutation answerOutcomeRelatedQuestion($data:OutcomeRelatedQAInput){answerOutcomeRelatedQuestion(data:$data${
            actions.obj ? "," + actions.obj : ""
          }){${actions.query.map(v => v + " ")}}}`,
          { data: actions.item },
          url,
          actions
        );
        yield put({
          type: WFME_API.OUTCOME_FORM.PUT.SUCCESS,
          data: response.data.data.answerOutcomeRelatedQuestion
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* del(actions) {
  yield call(loading);
  try {
    switch (actions.doc) {
      case KPI:
        var response = yield call(graphql.delete, "Kpi", url, actions);
        yield put({
          type: WFME_API.KPI.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/kpi"
        });
      case COURSE:
        var response = yield call(graphql.delete, "Course", url, actions);
        yield put({
          type: WFME_API.COURSE.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/course"
        });
      case COURSE_OFFER:
        var response = yield call(graphql.delete, "CourseOffer", url, actions);
        yield put({
          type: WFME_API.COURSE_OFFER.DEL.SUCCESS,
          data: actions.id
        });
        yield call(callback, true);
        return yield put({
          type: REDIRECT_TO,
          redirectTo: "/course/info?id=" + actions.item.courseId
        });
      case ANSWER:
        var response = yield call(
          graphql.delete,
          "CourseOfferKpiAnswer",
          url,
          actions
        );
        yield put({
          type: WFME_API.ANSWER.DEL.SUCCESS,
          data: actions.id
        });
        return yield call(callback, true);
      case FILE:
        var response = yield call(
          graphql.del,
          `/wfme/courseOffers/${actions.item.offerId}/kpiFileName/${
            actions.item.fileNameId
          }/file/${actions.item.fileId}`
        );
        yield put({
          type: WFME_REQUEST,
          doc: COURSE_OFFER,
          api: GET,
          item: {},
          id: actions.item.offerId,
          query: query.wfme.offer
        });
        return yield call(callback, true);
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
function* search(actions) {
  try {
    switch (actions.doc) {
      default:
        null;
    }
  } catch (e) {
    console.log(e);
    yield call(callback, false, e);
  }
}
export function* watchWFME() {
  yield takeEvery(WFME_REQUEST, request);
}
