import agent from "./agent";
import axios from "axios";
const convert = item => {
  return JSON.stringify(item).replace(/\"([^(\")"]+)\":/g, "$1:");
};
export default {
  get: (methods, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `query {${methods}{${actions.query.map(v => v + " ")}}}`
    });
  },
  filter: (methods, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `query ${methods}($query:Object){${methods}(query:$query){${actions.query.map(
        v => v + " "
      )}}}`,
      variables: { query: actions.item }
    });
  },
  search: (methods, skip = 0, limit = 10, url, actions, sort) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `query ${methods}($query:Object){${methods}(size:${limit},page:${skip}${
        sort ? `,sort:${sort}` : ""
      },query:$query){${actions.query.map(v => v + " ")}}}`,
      variables: { query: actions.item }
    });
  },
  post: (name, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `mutation insert${name}($data:${name}Input){insert${name}(data:$data${
        actions.obj ? "," + actions.obj : ""
      }){${actions.query.map(v => v + " ")}}}`,
      variables: { data: actions.item }
    });
  },
  put: (name, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `mutation update${name}($data:${name}Input){update${name}(data:$data,id:"${
        actions.id
      }"){${actions.query.map(v => v + " ")}}}`,
      variables: { data: actions.item }
    });
  },
  sqlPut: (name, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `mutation update${name}($data:${name}Input){update${name}(data:$data,id:${
        actions.id
      }){${actions.query.map(v => v + " ")}}}`,
      variables: { data: actions.item }
    });
  },
  custom: (name, method, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `mutation {${name}(${method}){${actions.query.map(
        v => v + " "
      )}}}`,
      variables: { data: actions.item }
    });
  },
  graph: (query, variables, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: query,
      variables: variables
    });
  },
  delete: (name, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `mutation delete${name}($id:ObjectId!){delete${name}(id:$id)}`,
      variables: { id: actions.id }
    });
  },
  sqlDelete: (name, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `mutation {delete${name}(id:${actions.id})}`,
      variables: {}
    });
  },
  clone: (name, url, actions) => {
    return agent(
      actions ? (actions.obj ? actions.obj.token : undefined) : undefined
    ).post(`/${url}/graphql`, {
      query: `mutation {${name}{${actions.query.map(v => v + " ")}}}`,
      variables: { data: actions.item }
    });
  },
  getRest: (url, item) => {
    return agent().get(url, {
      params: item
    });
  },
  postRest: (url, item) => {
    return agent().post(url, item);
  },
  putRest: (url, item) => {
    return agent().put(url, item);
  },
  import: (url, item) => {
    return agent().post(url, item);
  },
  update: (url, item) => {
    return agent().put(url, item);
  },
  postSpecial: (url, item) => {
    return axios({ url: url, type: "POST", data: "JSON" });
  },
  upload: (url, item) => {
    try {
      let data = new FormData();
      data.append("file", item.file, item.file.fileName);
      return axios
        .post("/api" + url, data, {
          headers: {
            accept: "application/json",
            "Accept-Language": "en-US,en;q=0.8",
            "Content-Type": `multipart/form-data; boundary=${data._boundary}`
          }
        })
        .then(response => {
          console.log(response);
          //handle success
        })
        .catch(error => {
          console.log(error);
          //handle error
        });
    } catch (e) {
      console.log(e);
    }
  },
  del: url => {
    return agent().delete(url);
  }
};
